#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 0.5.0.5149 (http://hl7.org/fhir/StructureDefinition/ElementDefinition) on 2015-07-06.
#  2015, SMART Health IT.


from . import address
from . import attachment
from . import codeableconcept
from . import coding
from . import contactpoint
from . import fhirdate
from . import fhirelement
from . import fhirreference
from . import humanname
from . import identifier
from . import period
from . import quantity
from . import range
from . import ratio
from . import signature
from . import timing


class ElementDefinition(fhirelement.FHIRElement):
    """ Definition of an elements in a resource or extension.
    
    Captures constraints on each element within the resource, profile, or
    extension.
    """
    
    resource_name = "ElementDefinition"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.alias = None
        """ Other names.
        List of `str` items. """
        
        self.binding = None
        """ ValueSet details if this is coded.
        Type `ElementDefinitionBinding` (represented as `dict` in JSON). """
        
        self.code = None
        """ Defining code.
        List of `Coding` items (represented as `dict` in JSON). """
        
        self.comments = None
        """ Comments about the use of this element.
        Type `str`. """
        
        self.condition = None
        """ Reference to invariant about presence.
        List of `str` items. """
        
        self.constraint = None
        """ Condition that must evaluate to true.
        List of `ElementDefinitionConstraint` items (represented as `dict` in JSON). """
        
        self.defaultValueAddress = None
        """ Specified value it missing from instance.
        Type `Address` (represented as `dict` in JSON). """
        
        self.defaultValueAttachment = None
        """ Specified value it missing from instance.
        Type `Attachment` (represented as `dict` in JSON). """
        
        self.defaultValueBase64Binary = None
        """ Specified value it missing from instance.
        Type `str`. """
        
        self.defaultValueBoolean = None
        """ Specified value it missing from instance.
        Type `bool`. """
        
        self.defaultValueCode = None
        """ Specified value it missing from instance.
        Type `str`. """
        
        self.defaultValueCodeableConcept = None
        """ Specified value it missing from instance.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.defaultValueCoding = None
        """ Specified value it missing from instance.
        Type `Coding` (represented as `dict` in JSON). """
        
        self.defaultValueContactPoint = None
        """ Specified value it missing from instance.
        Type `ContactPoint` (represented as `dict` in JSON). """
        
        self.defaultValueDate = None
        """ Specified value it missing from instance.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.defaultValueDateTime = None
        """ Specified value it missing from instance.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.defaultValueDecimal = None
        """ Specified value it missing from instance.
        Type `float`. """
        
        self.defaultValueHumanName = None
        """ Specified value it missing from instance.
        Type `HumanName` (represented as `dict` in JSON). """
        
        self.defaultValueIdentifier = None
        """ Specified value it missing from instance.
        Type `Identifier` (represented as `dict` in JSON). """
        
        self.defaultValueInstant = None
        """ Specified value it missing from instance.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.defaultValueInteger = None
        """ Specified value it missing from instance.
        Type `int`. """
        
        self.defaultValuePeriod = None
        """ Specified value it missing from instance.
        Type `Period` (represented as `dict` in JSON). """
        
        self.defaultValueQuantity = None
        """ Specified value it missing from instance.
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.defaultValueRange = None
        """ Specified value it missing from instance.
        Type `Range` (represented as `dict` in JSON). """
        
        self.defaultValueRatio = None
        """ Specified value it missing from instance.
        Type `Ratio` (represented as `dict` in JSON). """
        
        self.defaultValueReference = None
        """ Specified value it missing from instance.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.defaultValueSignature = None
        """ Specified value it missing from instance.
        Type `Signature` (represented as `dict` in JSON). """
        
        self.defaultValueString = None
        """ Specified value it missing from instance.
        Type `str`. """
        
        self.defaultValueTime = None
        """ Specified value it missing from instance.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.defaultValueTiming = None
        """ Specified value it missing from instance.
        Type `Timing` (represented as `dict` in JSON). """
        
        self.defaultValueUri = None
        """ Specified value it missing from instance.
        Type `str`. """
        
        self.definition = None
        """ Full formal definition as narrative text.
        Type `str`. """
        
        self.exampleAddress = None
        """ Example value: [as defined for type].
        Type `Address` (represented as `dict` in JSON). """
        
        self.exampleAttachment = None
        """ Example value: [as defined for type].
        Type `Attachment` (represented as `dict` in JSON). """
        
        self.exampleBase64Binary = None
        """ Example value: [as defined for type].
        Type `str`. """
        
        self.exampleBoolean = None
        """ Example value: [as defined for type].
        Type `bool`. """
        
        self.exampleCode = None
        """ Example value: [as defined for type].
        Type `str`. """
        
        self.exampleCodeableConcept = None
        """ Example value: [as defined for type].
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.exampleCoding = None
        """ Example value: [as defined for type].
        Type `Coding` (represented as `dict` in JSON). """
        
        self.exampleContactPoint = None
        """ Example value: [as defined for type].
        Type `ContactPoint` (represented as `dict` in JSON). """
        
        self.exampleDate = None
        """ Example value: [as defined for type].
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.exampleDateTime = None
        """ Example value: [as defined for type].
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.exampleDecimal = None
        """ Example value: [as defined for type].
        Type `float`. """
        
        self.exampleHumanName = None
        """ Example value: [as defined for type].
        Type `HumanName` (represented as `dict` in JSON). """
        
        self.exampleIdentifier = None
        """ Example value: [as defined for type].
        Type `Identifier` (represented as `dict` in JSON). """
        
        self.exampleInstant = None
        """ Example value: [as defined for type].
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.exampleInteger = None
        """ Example value: [as defined for type].
        Type `int`. """
        
        self.examplePeriod = None
        """ Example value: [as defined for type].
        Type `Period` (represented as `dict` in JSON). """
        
        self.exampleQuantity = None
        """ Example value: [as defined for type].
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.exampleRange = None
        """ Example value: [as defined for type].
        Type `Range` (represented as `dict` in JSON). """
        
        self.exampleRatio = None
        """ Example value: [as defined for type].
        Type `Ratio` (represented as `dict` in JSON). """
        
        self.exampleReference = None
        """ Example value: [as defined for type].
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.exampleSignature = None
        """ Example value: [as defined for type].
        Type `Signature` (represented as `dict` in JSON). """
        
        self.exampleString = None
        """ Example value: [as defined for type].
        Type `str`. """
        
        self.exampleTime = None
        """ Example value: [as defined for type].
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.exampleTiming = None
        """ Example value: [as defined for type].
        Type `Timing` (represented as `dict` in JSON). """
        
        self.exampleUri = None
        """ Example value: [as defined for type].
        Type `str`. """
        
        self.fixedAddress = None
        """ Value must be exactly this.
        Type `Address` (represented as `dict` in JSON). """
        
        self.fixedAttachment = None
        """ Value must be exactly this.
        Type `Attachment` (represented as `dict` in JSON). """
        
        self.fixedBase64Binary = None
        """ Value must be exactly this.
        Type `str`. """
        
        self.fixedBoolean = None
        """ Value must be exactly this.
        Type `bool`. """
        
        self.fixedCode = None
        """ Value must be exactly this.
        Type `str`. """
        
        self.fixedCodeableConcept = None
        """ Value must be exactly this.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.fixedCoding = None
        """ Value must be exactly this.
        Type `Coding` (represented as `dict` in JSON). """
        
        self.fixedContactPoint = None
        """ Value must be exactly this.
        Type `ContactPoint` (represented as `dict` in JSON). """
        
        self.fixedDate = None
        """ Value must be exactly this.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.fixedDateTime = None
        """ Value must be exactly this.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.fixedDecimal = None
        """ Value must be exactly this.
        Type `float`. """
        
        self.fixedHumanName = None
        """ Value must be exactly this.
        Type `HumanName` (represented as `dict` in JSON). """
        
        self.fixedIdentifier = None
        """ Value must be exactly this.
        Type `Identifier` (represented as `dict` in JSON). """
        
        self.fixedInstant = None
        """ Value must be exactly this.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.fixedInteger = None
        """ Value must be exactly this.
        Type `int`. """
        
        self.fixedPeriod = None
        """ Value must be exactly this.
        Type `Period` (represented as `dict` in JSON). """
        
        self.fixedQuantity = None
        """ Value must be exactly this.
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.fixedRange = None
        """ Value must be exactly this.
        Type `Range` (represented as `dict` in JSON). """
        
        self.fixedRatio = None
        """ Value must be exactly this.
        Type `Ratio` (represented as `dict` in JSON). """
        
        self.fixedReference = None
        """ Value must be exactly this.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.fixedSignature = None
        """ Value must be exactly this.
        Type `Signature` (represented as `dict` in JSON). """
        
        self.fixedString = None
        """ Value must be exactly this.
        Type `str`. """
        
        self.fixedTime = None
        """ Value must be exactly this.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.fixedTiming = None
        """ Value must be exactly this.
        Type `Timing` (represented as `dict` in JSON). """
        
        self.fixedUri = None
        """ Value must be exactly this.
        Type `str`. """
        
        self.isModifier = None
        """ If this modifies the meaning of other elements.
        Type `bool`. """
        
        self.isSummary = None
        """ Include when _summary = true?.
        Type `bool`. """
        
        self.label = None
        """ Name for element to display with or prompt for element.
        Type `str`. """
        
        self.mapping = None
        """ Map element to another set of definitions.
        List of `ElementDefinitionMapping` items (represented as `dict` in JSON). """
        
        self.max = None
        """ Maximum Cardinality (a number or *).
        Type `str`. """
        
        self.maxLength = None
        """ Max length for strings.
        Type `int`. """
        
        self.meaningWhenMissing = None
        """ Implicit meaning when this element is missing.
        Type `str`. """
        
        self.min = None
        """ Minimum Cardinality.
        Type `int`. """
        
        self.mustSupport = None
        """ If the element must supported.
        Type `bool`. """
        
        self.name = None
        """ Name for this particular element definition (reference target).
        Type `str`. """
        
        self.nameReference = None
        """ To another element constraint (by element.name).
        Type `str`. """
        
        self.path = None
        """ The path of the element (see the Detailed Descriptions).
        Type `str`. """
        
        self.patternAddress = None
        """ Value must have at least these property values.
        Type `Address` (represented as `dict` in JSON). """
        
        self.patternAttachment = None
        """ Value must have at least these property values.
        Type `Attachment` (represented as `dict` in JSON). """
        
        self.patternBase64Binary = None
        """ Value must have at least these property values.
        Type `str`. """
        
        self.patternBoolean = None
        """ Value must have at least these property values.
        Type `bool`. """
        
        self.patternCode = None
        """ Value must have at least these property values.
        Type `str`. """
        
        self.patternCodeableConcept = None
        """ Value must have at least these property values.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.patternCoding = None
        """ Value must have at least these property values.
        Type `Coding` (represented as `dict` in JSON). """
        
        self.patternContactPoint = None
        """ Value must have at least these property values.
        Type `ContactPoint` (represented as `dict` in JSON). """
        
        self.patternDate = None
        """ Value must have at least these property values.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.patternDateTime = None
        """ Value must have at least these property values.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.patternDecimal = None
        """ Value must have at least these property values.
        Type `float`. """
        
        self.patternHumanName = None
        """ Value must have at least these property values.
        Type `HumanName` (represented as `dict` in JSON). """
        
        self.patternIdentifier = None
        """ Value must have at least these property values.
        Type `Identifier` (represented as `dict` in JSON). """
        
        self.patternInstant = None
        """ Value must have at least these property values.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.patternInteger = None
        """ Value must have at least these property values.
        Type `int`. """
        
        self.patternPeriod = None
        """ Value must have at least these property values.
        Type `Period` (represented as `dict` in JSON). """
        
        self.patternQuantity = None
        """ Value must have at least these property values.
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.patternRange = None
        """ Value must have at least these property values.
        Type `Range` (represented as `dict` in JSON). """
        
        self.patternRatio = None
        """ Value must have at least these property values.
        Type `Ratio` (represented as `dict` in JSON). """
        
        self.patternReference = None
        """ Value must have at least these property values.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.patternSignature = None
        """ Value must have at least these property values.
        Type `Signature` (represented as `dict` in JSON). """
        
        self.patternString = None
        """ Value must have at least these property values.
        Type `str`. """
        
        self.patternTime = None
        """ Value must have at least these property values.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.patternTiming = None
        """ Value must have at least these property values.
        Type `Timing` (represented as `dict` in JSON). """
        
        self.patternUri = None
        """ Value must have at least these property values.
        Type `str`. """
        
        self.representation = None
        """ How this element is represented in instances.
        List of `str` items. """
        
        self.requirements = None
        """ Why is this needed?.
        Type `str`. """
        
        self.short = None
        """ Concise definition for xml presentation.
        Type `str`. """
        
        self.slicing = None
        """ This element is sliced - slices follow.
        Type `ElementDefinitionSlicing` (represented as `dict` in JSON). """
        
        self.type = None
        """ Data type and Profile for this element.
        List of `ElementDefinitionType` items (represented as `dict` in JSON). """
        
        super(ElementDefinition, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(ElementDefinition, self).elementProperties()
        js.extend([
            ("alias", "alias", str, True),
            ("binding", "binding", ElementDefinitionBinding, False),
            ("code", "code", coding.Coding, True),
            ("comments", "comments", str, False),
            ("condition", "condition", str, True),
            ("constraint", "constraint", ElementDefinitionConstraint, True),
            ("defaultValueAddress", "defaultValueAddress", address.Address, False),
            ("defaultValueAttachment", "defaultValueAttachment", attachment.Attachment, False),
            ("defaultValueBase64Binary", "defaultValueBase64Binary", str, False),
            ("defaultValueBoolean", "defaultValueBoolean", bool, False),
            ("defaultValueCode", "defaultValueCode", str, False),
            ("defaultValueCodeableConcept", "defaultValueCodeableConcept", codeableconcept.CodeableConcept, False),
            ("defaultValueCoding", "defaultValueCoding", coding.Coding, False),
            ("defaultValueContactPoint", "defaultValueContactPoint", contactpoint.ContactPoint, False),
            ("defaultValueDate", "defaultValueDate", fhirdate.FHIRDate, False),
            ("defaultValueDateTime", "defaultValueDateTime", fhirdate.FHIRDate, False),
            ("defaultValueDecimal", "defaultValueDecimal", float, False),
            ("defaultValueHumanName", "defaultValueHumanName", humanname.HumanName, False),
            ("defaultValueIdentifier", "defaultValueIdentifier", identifier.Identifier, False),
            ("defaultValueInstant", "defaultValueInstant", fhirdate.FHIRDate, False),
            ("defaultValueInteger", "defaultValueInteger", int, False),
            ("defaultValuePeriod", "defaultValuePeriod", period.Period, False),
            ("defaultValueQuantity", "defaultValueQuantity", quantity.Quantity, False),
            ("defaultValueRange", "defaultValueRange", range.Range, False),
            ("defaultValueRatio", "defaultValueRatio", ratio.Ratio, False),
            ("defaultValueReference", "defaultValueReference", fhirreference.FHIRReference, False),
            ("defaultValueSignature", "defaultValueSignature", signature.Signature, False),
            ("defaultValueString", "defaultValueString", str, False),
            ("defaultValueTime", "defaultValueTime", fhirdate.FHIRDate, False),
            ("defaultValueTiming", "defaultValueTiming", timing.Timing, False),
            ("defaultValueUri", "defaultValueUri", str, False),
            ("definition", "definition", str, False),
            ("exampleAddress", "exampleAddress", address.Address, False),
            ("exampleAttachment", "exampleAttachment", attachment.Attachment, False),
            ("exampleBase64Binary", "exampleBase64Binary", str, False),
            ("exampleBoolean", "exampleBoolean", bool, False),
            ("exampleCode", "exampleCode", str, False),
            ("exampleCodeableConcept", "exampleCodeableConcept", codeableconcept.CodeableConcept, False),
            ("exampleCoding", "exampleCoding", coding.Coding, False),
            ("exampleContactPoint", "exampleContactPoint", contactpoint.ContactPoint, False),
            ("exampleDate", "exampleDate", fhirdate.FHIRDate, False),
            ("exampleDateTime", "exampleDateTime", fhirdate.FHIRDate, False),
            ("exampleDecimal", "exampleDecimal", float, False),
            ("exampleHumanName", "exampleHumanName", humanname.HumanName, False),
            ("exampleIdentifier", "exampleIdentifier", identifier.Identifier, False),
            ("exampleInstant", "exampleInstant", fhirdate.FHIRDate, False),
            ("exampleInteger", "exampleInteger", int, False),
            ("examplePeriod", "examplePeriod", period.Period, False),
            ("exampleQuantity", "exampleQuantity", quantity.Quantity, False),
            ("exampleRange", "exampleRange", range.Range, False),
            ("exampleRatio", "exampleRatio", ratio.Ratio, False),
            ("exampleReference", "exampleReference", fhirreference.FHIRReference, False),
            ("exampleSignature", "exampleSignature", signature.Signature, False),
            ("exampleString", "exampleString", str, False),
            ("exampleTime", "exampleTime", fhirdate.FHIRDate, False),
            ("exampleTiming", "exampleTiming", timing.Timing, False),
            ("exampleUri", "exampleUri", str, False),
            ("fixedAddress", "fixedAddress", address.Address, False),
            ("fixedAttachment", "fixedAttachment", attachment.Attachment, False),
            ("fixedBase64Binary", "fixedBase64Binary", str, False),
            ("fixedBoolean", "fixedBoolean", bool, False),
            ("fixedCode", "fixedCode", str, False),
            ("fixedCodeableConcept", "fixedCodeableConcept", codeableconcept.CodeableConcept, False),
            ("fixedCoding", "fixedCoding", coding.Coding, False),
            ("fixedContactPoint", "fixedContactPoint", contactpoint.ContactPoint, False),
            ("fixedDate", "fixedDate", fhirdate.FHIRDate, False),
            ("fixedDateTime", "fixedDateTime", fhirdate.FHIRDate, False),
            ("fixedDecimal", "fixedDecimal", float, False),
            ("fixedHumanName", "fixedHumanName", humanname.HumanName, False),
            ("fixedIdentifier", "fixedIdentifier", identifier.Identifier, False),
            ("fixedInstant", "fixedInstant", fhirdate.FHIRDate, False),
            ("fixedInteger", "fixedInteger", int, False),
            ("fixedPeriod", "fixedPeriod", period.Period, False),
            ("fixedQuantity", "fixedQuantity", quantity.Quantity, False),
            ("fixedRange", "fixedRange", range.Range, False),
            ("fixedRatio", "fixedRatio", ratio.Ratio, False),
            ("fixedReference", "fixedReference", fhirreference.FHIRReference, False),
            ("fixedSignature", "fixedSignature", signature.Signature, False),
            ("fixedString", "fixedString", str, False),
            ("fixedTime", "fixedTime", fhirdate.FHIRDate, False),
            ("fixedTiming", "fixedTiming", timing.Timing, False),
            ("fixedUri", "fixedUri", str, False),
            ("isModifier", "isModifier", bool, False),
            ("isSummary", "isSummary", bool, False),
            ("label", "label", str, False),
            ("mapping", "mapping", ElementDefinitionMapping, True),
            ("max", "max", str, False),
            ("maxLength", "maxLength", int, False),
            ("meaningWhenMissing", "meaningWhenMissing", str, False),
            ("min", "min", int, False),
            ("mustSupport", "mustSupport", bool, False),
            ("name", "name", str, False),
            ("nameReference", "nameReference", str, False),
            ("path", "path", str, False),
            ("patternAddress", "patternAddress", address.Address, False),
            ("patternAttachment", "patternAttachment", attachment.Attachment, False),
            ("patternBase64Binary", "patternBase64Binary", str, False),
            ("patternBoolean", "patternBoolean", bool, False),
            ("patternCode", "patternCode", str, False),
            ("patternCodeableConcept", "patternCodeableConcept", codeableconcept.CodeableConcept, False),
            ("patternCoding", "patternCoding", coding.Coding, False),
            ("patternContactPoint", "patternContactPoint", contactpoint.ContactPoint, False),
            ("patternDate", "patternDate", fhirdate.FHIRDate, False),
            ("patternDateTime", "patternDateTime", fhirdate.FHIRDate, False),
            ("patternDecimal", "patternDecimal", float, False),
            ("patternHumanName", "patternHumanName", humanname.HumanName, False),
            ("patternIdentifier", "patternIdentifier", identifier.Identifier, False),
            ("patternInstant", "patternInstant", fhirdate.FHIRDate, False),
            ("patternInteger", "patternInteger", int, False),
            ("patternPeriod", "patternPeriod", period.Period, False),
            ("patternQuantity", "patternQuantity", quantity.Quantity, False),
            ("patternRange", "patternRange", range.Range, False),
            ("patternRatio", "patternRatio", ratio.Ratio, False),
            ("patternReference", "patternReference", fhirreference.FHIRReference, False),
            ("patternSignature", "patternSignature", signature.Signature, False),
            ("patternString", "patternString", str, False),
            ("patternTime", "patternTime", fhirdate.FHIRDate, False),
            ("patternTiming", "patternTiming", timing.Timing, False),
            ("patternUri", "patternUri", str, False),
            ("representation", "representation", str, True),
            ("requirements", "requirements", str, False),
            ("short", "short", str, False),
            ("slicing", "slicing", ElementDefinitionSlicing, False),
            ("type", "type", ElementDefinitionType, True),
        ])
        return js


class ElementDefinitionBinding(fhirelement.FHIRElement):
    """ ValueSet details if this is coded.
    
    Binds to a value set if this element is coded (code, Coding,
    CodeableConcept).
    """
    
    resource_name = "ElementDefinitionBinding"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.description = None
        """ Human explanation of the value set.
        Type `str`. """
        
        self.name = None
        """ Descriptive Name.
        Type `str`. """
        
        self.strength = None
        """ required | extensible | preferred | example.
        Type `str`. """
        
        self.valueSetReference = None
        """ Source of value set.
        Type `FHIRReference` referencing `ValueSet` (represented as `dict` in JSON). """
        
        self.valueSetUri = None
        """ Source of value set.
        Type `str`. """
        
        super(ElementDefinitionBinding, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(ElementDefinitionBinding, self).elementProperties()
        js.extend([
            ("description", "description", str, False),
            ("name", "name", str, False),
            ("strength", "strength", str, False),
            ("valueSetReference", "valueSetReference", fhirreference.FHIRReference, False),
            ("valueSetUri", "valueSetUri", str, False),
        ])
        return js


class ElementDefinitionConstraint(fhirelement.FHIRElement):
    """ Condition that must evaluate to true.
    
    Formal constraints such as co-occurrence and other constraints that can be
    computationally evaluated within the context of the instance.
    """
    
    resource_name = "ElementDefinitionConstraint"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.human = None
        """ Human description of constraint.
        Type `str`. """
        
        self.key = None
        """ Target of 'condition' reference above.
        Type `str`. """
        
        self.name = None
        """ Short human label.
        Type `str`. """
        
        self.severity = None
        """ error | warning.
        Type `str`. """
        
        self.xpath = None
        """ XPath expression of constraint.
        Type `str`. """
        
        super(ElementDefinitionConstraint, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(ElementDefinitionConstraint, self).elementProperties()
        js.extend([
            ("human", "human", str, False),
            ("key", "key", str, False),
            ("name", "name", str, False),
            ("severity", "severity", str, False),
            ("xpath", "xpath", str, False),
        ])
        return js


class ElementDefinitionMapping(fhirelement.FHIRElement):
    """ Map element to another set of definitions.
    
    Identifies a concept from an external specification that roughly
    corresponds to this element.
    """
    
    resource_name = "ElementDefinitionMapping"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.identity = None
        """ Reference to mapping declaration.
        Type `str`. """
        
        self.language = None
        """ Computable language of mapping.
        Type `str`. """
        
        self.map = None
        """ Details of the mapping.
        Type `str`. """
        
        super(ElementDefinitionMapping, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(ElementDefinitionMapping, self).elementProperties()
        js.extend([
            ("identity", "identity", str, False),
            ("language", "language", str, False),
            ("map", "map", str, False),
        ])
        return js


class ElementDefinitionSlicing(fhirelement.FHIRElement):
    """ This element is sliced - slices follow.
    
    Indicates that the element is sliced into a set of alternative definitions
    (there are multiple definitions on a single element in the base resource).
    The set of slices is any elements that come after this in the element
    sequence that have the same path, until a shorter path occurs (the shorter
    path terminates the set).
    """
    
    resource_name = "ElementDefinitionSlicing"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.description = None
        """ Text description of how slicing works (or not).
        Type `str`. """
        
        self.discriminator = None
        """ Element values that used to distinguish the slices.
        List of `str` items. """
        
        self.ordered = None
        """ If elements must be in same order as slices.
        Type `bool`. """
        
        self.rules = None
        """ closed | open | openAtEnd.
        Type `str`. """
        
        super(ElementDefinitionSlicing, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(ElementDefinitionSlicing, self).elementProperties()
        js.extend([
            ("description", "description", str, False),
            ("discriminator", "discriminator", str, True),
            ("ordered", "ordered", bool, False),
            ("rules", "rules", str, False),
        ])
        return js


class ElementDefinitionType(fhirelement.FHIRElement):
    """ Data type and Profile for this element.
    
    The data type or resource that the value of this element is permitted to
    be.
    """
    
    resource_name = "ElementDefinitionType"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.aggregation = None
        """ contained | referenced | bundled - how aggregated.
        List of `str` items. """
        
        self.code = None
        """ Name of Data type or Resource.
        Type `str`. """
        
        self.profile = None
        """ Profile.structure to apply.
        Type `str`. """
        
        super(ElementDefinitionType, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(ElementDefinitionType, self).elementProperties()
        js.extend([
            ("aggregation", "aggregation", str, True),
            ("code", "code", str, False),
            ("profile", "profile", str, False),
        ])
        return js




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 0.5.0.5149 (http://hl7.org/fhir/StructureDefinition/DeviceUseRequest) on 2015-07-06.
#  2015, SMART Health IT.


from . import codeableconcept
from . import domainresource
from . import fhirdate
from . import fhirreference
from . import identifier
from . import period
from . import timing


class DeviceUseRequest(domainresource.DomainResource):
    """ A request for a patient to use or be given a medical device.
    
    Represents a request for a patient to employ a medical device. The device
    may be an implantable device, or an external assistive device, such as a
    walker.
    """
    
    resource_name = "DeviceUseRequest"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.bodySiteCodeableConcept = None
        """ Target body site.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.bodySiteReference = None
        """ Target body site.
        Type `FHIRReference` referencing `BodySite` (represented as `dict` in JSON). """
        
        self.device = None
        """ Device requested.
        Type `FHIRReference` referencing `Device` (represented as `dict` in JSON). """
        
        self.encounter = None
        """ Encounter motivating request.
        Type `FHIRReference` referencing `Encounter` (represented as `dict` in JSON). """
        
        self.identifier = None
        """ Request identifier.
        List of `Identifier` items (represented as `dict` in JSON). """
        
        self.indication = None
        """ Reason for request.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.notes = None
        """ Notes or comments.
        List of `str` items. """
        
        self.orderedOn = None
        """ When ordered.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.priority = None
        """ routine | urgent | stat | asap.
        Type `str`. """
        
        self.prnReason = None
        """ PRN.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.recordedOn = None
        """ When recorded.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.status = None
        """ proposed | planned | requested | received | accepted | in-progress
        | completed | suspended | rejected | aborted.
        Type `str`. """
        
        self.subject = None
        """ Focus of request.
        Type `FHIRReference` referencing `Patient` (represented as `dict` in JSON). """
        
        self.timingDateTime = None
        """ Schedule for use.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.timingPeriod = None
        """ Schedule for use.
        Type `Period` (represented as `dict` in JSON). """
        
        self.timingTiming = None
        """ Schedule for use.
        Type `Timing` (represented as `dict` in JSON). """
        
        super(DeviceUseRequest, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(DeviceUseRequest, self).elementProperties()
        js.extend([
            ("bodySiteCodeableConcept", "bodySiteCodeableConcept", codeableconcept.CodeableConcept, False),
            ("bodySiteReference", "bodySiteReference", fhirreference.FHIRReference, False),
            ("device", "device", fhirreference.FHIRReference, False),
            ("encounter", "encounter", fhirreference.FHIRReference, False),
            ("identifier", "identifier", identifier.Identifier, True),
            ("indication", "indication", codeableconcept.CodeableConcept, True),
            ("notes", "notes", str, True),
            ("orderedOn", "orderedOn", fhirdate.FHIRDate, False),
            ("priority", "priority", str, False),
            ("prnReason", "prnReason", codeableconcept.CodeableConcept, True),
            ("recordedOn", "recordedOn", fhirdate.FHIRDate, False),
            ("status", "status", str, False),
            ("subject", "subject", fhirreference.FHIRReference, False),
            ("timingDateTime", "timingDateTime", fhirdate.FHIRDate, False),
            ("timingPeriod", "timingPeriod", period.Period, False),
            ("timingTiming", "timingTiming", timing.Timing, False),
        ])
        return js




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 0.5.0.5149 on 2015-07-06.
#  2015, SMART Health IT.


import os
import io
import unittest
import json
from . import orderresponse
from .fhirdate import FHIRDate


class OrderResponseTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("OrderResponse", js["resourceType"])
        return orderresponse.OrderResponse(js)
    
    def testOrderResponse1(self):
        inst = self.instantiate_from("orderresponse-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a OrderResponse instance")
        self.implOrderResponse1(inst)
        
        js = inst.as_json()
        self.assertEqual("OrderResponse", js["resourceType"])
        inst2 = orderresponse.OrderResponse(js)
        self.implOrderResponse1(inst2)
    
    def implOrderResponse1(self, inst):
        self.assertEqual(inst.date.date, FHIRDate("2012-12-28T13:10:56+11:00").date)
        self.assertEqual(inst.date.as_json(), "2012-12-28T13:10:56+11:00")
        self.assertEqual(inst.id, "example")
        self.assertEqual(inst.orderStatus, "completed")
        self.assertEqual(inst.text.div, "<div>Lab Report completed at 13:10 28-Dec 2012</div>")
        self.assertEqual(inst.text.status, "generated")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 0.5.0.5149 (http://hl7.org/fhir/StructureDefinition/Bundle) on 2015-07-06.
#  2015, SMART Health IT.


from . import fhirdate
from . import fhirelement
from . import resource


class Bundle(resource.Resource):
    """ Contains a collection of resources.
    
    A container for a collection of resources.
    """
    
    resource_name = "Bundle"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.base = None
        """ Stated Base URL.
        Type `str`. """
        
        self.entry = None
        """ Entry in the bundle - will have a resource, or information.
        List of `BundleEntry` items (represented as `dict` in JSON). """
        
        self.link = None
        """ Links related to this Bundle.
        List of `BundleLink` items (represented as `dict` in JSON). """
        
        self.signature = None
        """ XML Digital Signature (base64 encoded).
        Type `str`. """
        
        self.total = None
        """ If search, the total number of matches.
        Type `int`. """
        
        self.type = None
        """ document | message | transaction | transaction-response | history |
        searchset | collection.
        Type `str`. """
        
        super(Bundle, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(Bundle, self).elementProperties()
        js.extend([
            ("base", "base", str, False),
            ("entry", "entry", BundleEntry, True),
            ("link", "link", BundleLink, True),
            ("signature", "signature", str, False),
            ("total", "total", int, False),
            ("type", "type", str, False),
        ])
        return js


class BundleEntry(fhirelement.FHIRElement):
    """ Entry in the bundle - will have a resource, or information.
    
    An entry in a bundle resource - will either contain a resource, or
    information about a resource (transactions and history only).
    """
    
    resource_name = "BundleEntry"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.base = None
        """ Base URL, if different to bundle base.
        Type `str`. """
        
        self.link = None
        """ Links related to this entry.
        List of `BundleLink` items (represented as `dict` in JSON). """
        
        self.resource = None
        """ Resources in this bundle.
        Type `Resource` (represented as `dict` in JSON). """
        
        self.search = None
        """ Search related information.
        Type `BundleEntrySearch` (represented as `dict` in JSON). """
        
        self.transaction = None
        """ Transaction Related Information.
        Type `BundleEntryTransaction` (represented as `dict` in JSON). """
        
        self.transactionResponse = None
        """ Transaction Related Information.
        Type `BundleEntryTransactionResponse` (represented as `dict` in JSON). """
        
        super(BundleEntry, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(BundleEntry, self).elementProperties()
        js.extend([
            ("base", "base", str, False),
            ("link", "link", BundleLink, True),
            ("resource", "resource", resource.Resource, False),
            ("search", "search", BundleEntrySearch, False),
            ("transaction", "transaction", BundleEntryTransaction, False),
            ("transactionResponse", "transactionResponse", BundleEntryTransactionResponse, False),
        ])
        return js


class BundleEntrySearch(fhirelement.FHIRElement):
    """ Search related information.
    
    Information about the search process that lead to the creation of this
    entry.
    """
    
    resource_name = "BundleEntrySearch"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.mode = None
        """ match | include - why this is in the result set.
        Type `str`. """
        
        self.score = None
        """ Search ranking (between 0 and 1).
        Type `float`. """
        
        super(BundleEntrySearch, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(BundleEntrySearch, self).elementProperties()
        js.extend([
            ("mode", "mode", str, False),
            ("score", "score", float, False),
        ])
        return js


class BundleEntryTransaction(fhirelement.FHIRElement):
    """ Transaction Related Information.
    
    Additional information about how this entry should be processed as part of
    a transaction.
    """
    
    resource_name = "BundleEntryTransaction"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.ifMatch = None
        """ For managing update contention.
        Type `str`. """
        
        self.ifModifiedSince = None
        """ For managing update contention.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.ifNoneExist = None
        """ For conditional creates.
        Type `str`. """
        
        self.ifNoneMatch = None
        """ For managing cache currency.
        Type `str`. """
        
        self.method = None
        """ GET | POST | PUT | DELETE.
        Type `str`. """
        
        self.url = None
        """ The URL for the transaction.
        Type `str`. """
        
        super(BundleEntryTransaction, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(BundleEntryTransaction, self).elementProperties()
        js.extend([
            ("ifMatch", "ifMatch", str, False),
            ("ifModifiedSince", "ifModifiedSince", fhirdate.FHIRDate, False),
            ("ifNoneExist", "ifNoneExist", str, False),
            ("ifNoneMatch", "ifNoneMatch", str, False),
            ("method", "method", str, False),
            ("url", "url", str, False),
        ])
        return js


class BundleEntryTransactionResponse(fhirelement.FHIRElement):
    """ Transaction Related Information.
    
    Additional information about how this entry should be processed as part of
    a transaction.
    """
    
    resource_name = "BundleEntryTransactionResponse"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.etag = None
        """ The etag for the resource (if relevant).
        Type `str`. """
        
        self.lastModified = None
        """ Server's date time modified.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.location = None
        """ The location, if the operation returns a location.
        Type `str`. """
        
        self.status = None
        """ Status return code for entry.
        Type `str`. """
        
        super(BundleEntryTransactionResponse, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(BundleEntryTransactionResponse, self).elementProperties()
        js.extend([
            ("etag", "etag", str, False),
            ("lastModified", "lastModified", fhirdate.FHIRDate, False),
            ("location", "location", str, False),
            ("status", "status", str, False),
        ])
        return js


class BundleLink(fhirelement.FHIRElement):
    """ Links related to this Bundle.
    
    A series of links that provide context to this bundle.
    """
    
    resource_name = "BundleLink"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.relation = None
        """ http://www.iana.org/assignments/link-relations/link-relations.xhtml.
        Type `str`. """
        
        self.url = None
        """ Reference details for the link.
        Type `str`. """
        
        super(BundleLink, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(BundleLink, self).elementProperties()
        js.extend([
            ("relation", "relation", str, False),
            ("url", "url", str, False),
        ])
        return js




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 0.5.0.5149 on 2015-07-06.
#  2015, SMART Health IT.


import os
import io
import unittest
import json
from . import observation
from .fhirdate import FHIRDate


class ObservationTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("Observation", js["resourceType"])
        return observation.Observation(js)
    
    def testObservation1(self):
        inst = self.instantiate_from("obs-genetics-example1-somatic.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation1(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation1(inst2)
    
    def implObservation1(self, inst):
        self.assertEqual(inst.code.coding[0].code, "21665-5")
        self.assertEqual(inst.code.coding[0].display, "EGFR gene mutation analysis")
        self.assertEqual(inst.code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.comments, "The EGFR p.L858R mutation has been associated with response to anti-EGFR therapy")
        self.assertEqual(inst.extension[0].url, "http://hl7.org/fhir/StructureDefinition/geneticsGenomeBuild")
        self.assertEqual(inst.extension[0].valueString, "GRCh 37")
        self.assertEqual(inst.extension[1].url, "http://hl7.org/fhir/StructureDefinition/geneticsChromosome")
        self.assertEqual(inst.extension[1].valueString, "7")
        self.assertEqual(inst.extension[2].url, "http://hl7.org/fhir/StructureDefinition/geneticsGenomicReferenceSequenceId")
        self.assertEqual(inst.extension[2].valueCode, "NC_000007.13")
        self.assertEqual(inst.extension[3].url, "http://hl7.org/fhir/StructureDefinition/geneticsGenomicStart")
        self.assertEqual(inst.extension[3].valueInteger, 55259515)
        self.assertEqual(inst.extension[4].url, "http://hl7.org/fhir/StructureDefinition/geneticsGenomicStop")
        self.assertEqual(inst.extension[4].valueInteger, 55259515)
        self.assertEqual(inst.extension[5].url, "http://hl7.org/fhir/StructureDefinition/geneticsReferenceAllele")
        self.assertEqual(inst.extension[5].valueString, "T")
        self.assertEqual(inst.extension[6].url, "http://hl7.org/fhir/StructureDefinition/geneticsObservedAllele")
        self.assertEqual(inst.extension[6].valueString, "G")
        self.assertEqual(inst.extension[7].url, "http://hl7.org/fhir/StructureDefinition/geneticsGeneId")
        self.assertEqual(inst.extension[7].valueCodeableConcept.coding[0].code, "3236")
        self.assertEqual(inst.extension[7].valueCodeableConcept.coding[0].display, "EGFR")
        self.assertEqual(inst.extension[7].valueCodeableConcept.coding[0].system, "http://www.genenames.org")
        self.assertEqual(inst.extension[8].url, "http://hl7.org/fhir/StructureDefinition/geneticsTranscriptReferenceSequenseId")
        self.assertEqual(inst.extension[8].valueCode, "ENSESTT00000085772.1")
        self.assertEqual(inst.extension[9].url, "http://hl7.org/fhir/StructureDefinition/geneticsDNASequenceVariation")
        self.assertEqual(inst.extension[9].valueString, "c.2573T>G")
        self.assertEqual(inst.id, "genetics-example1-somatic")
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
    
    def testObservation2(self):
        inst = self.instantiate_from("obs-genetics-example2-germline.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation2(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation2(inst2)
    
    def implObservation2(self, inst):
        self.assertEqual(inst.code.coding[0].code, "21636-6")
        self.assertEqual(inst.code.coding[0].display, "BRCA1 gene mutation analysis")
        self.assertEqual(inst.code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.extension[0].url, "http://hl7.org/fhir/StructureDefinition/geneticsGenomeBuild")
        self.assertEqual(inst.extension[0].valueString, "GRCh 37")
        self.assertEqual(inst.extension[1].url, "http://hl7.org/fhir/StructureDefinition/geneticsChromosome")
        self.assertEqual(inst.extension[1].valueString, "17")
        self.assertEqual(inst.extension[2].url, "http://hl7.org/fhir/StructureDefinition/geneticsGenomicReferenceSequenceId")
        self.assertEqual(inst.extension[2].valueCode, "NC_000017.10")
        self.assertEqual(inst.extension[3].url, "http://hl7.org/fhir/StructureDefinition/geneticsGenomicStart")
        self.assertEqual(inst.extension[3].valueInteger, 41258504)
        self.assertEqual(inst.extension[4].url, "http://hl7.org/fhir/StructureDefinition/geneticsGenomicStop")
        self.assertEqual(inst.extension[4].valueInteger, 41258504)
        self.assertEqual(inst.extension[5].url, "http://hl7.org/fhir/StructureDefinition/geneticsReferenceAllele")
        self.assertEqual(inst.extension[5].valueString, "A")
        self.assertEqual(inst.extension[6].url, "http://hl7.org/fhir/StructureDefinition/geneticsObservedAllele")
        self.assertEqual(inst.extension[6].valueString, "C")
        self.assertEqual(inst.extension[7].url, "http://hl7.org/fhir/StructureDefinition/geneticsGeneId")
        self.assertEqual(inst.extension[7].valueCodeableConcept.coding[0].code, "1100")
        self.assertEqual(inst.extension[7].valueCodeableConcept.coding[0].display, "BRCA1")
        self.assertEqual(inst.extension[7].valueCodeableConcept.coding[0].system, "http://www.genenames.org")
        self.assertEqual(inst.extension[8].url, "http://hl7.org/fhir/StructureDefinition/geneticsTranscriptReferenceSequenseId")
        self.assertEqual(inst.extension[8].valueCode, "NM_007294.3")
        self.assertEqual(inst.extension[9].url, "http://hl7.org/fhir/StructureDefinition/geneticsDNASequenceVariation")
        self.assertEqual(inst.extension[9].valueString, "c.181T>G")
        self.assertEqual(inst.id, "genetics-example2-germline")
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
    
    def testObservation3(self):
        inst = self.instantiate_from("obs-uslab-example1.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation3(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation3(inst2)
    
    def implObservation3(self, inst):
        self.assertEqual(inst.appliesDateTime.date, FHIRDate("2014-12-03").date)
        self.assertEqual(inst.appliesDateTime.as_json(), "2014-12-03")
        self.assertEqual(inst.code.coding[0].code, "10368-9")
        self.assertEqual(inst.code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.code.coding[1].code, "CAPLD")
        self.assertEqual(inst.code.coding[1].display, "Blood Lead, Capillary")
        self.assertEqual(inst.code.coding[1].system, "urn:oid:2.16.840.1.113883.3.72.5.24")
        self.assertEqual(inst.code.text, "Capillary Blood Lead Screen")
        self.assertEqual(inst.extension[0].url, "http://hl7.org/fhir/StructureDefinition/uslabobservationkind")
        self.assertEqual(inst.extension[0].valueCode, "result")
        self.assertEqual(inst.id, "uslab-example1")
        self.assertEqual(inst.identifier[0].system, "http://lis.acmelabs.org/identifiers/labtestresult")
        self.assertEqual(inst.identifier[0].type.text, "lab test result ID")
        self.assertEqual(inst.identifier[0].use, "official")
        self.assertEqual(inst.identifier[0].value, "1234")
        self.assertEqual(inst.interpretation.coding[0].code, "H")
        self.assertEqual(inst.interpretation.coding[0].system, "http://hl7.org/fhir/v2/0078")
        self.assertEqual(inst.issued.date, FHIRDate("2014-12-04T15:42:15-08:00").date)
        self.assertEqual(inst.issued.as_json(), "2014-12-04T15:42:15-08:00")
        self.assertEqual(inst.referenceRange[0].high.code, "ug/dL")
        self.assertEqual(inst.referenceRange[0].high.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.referenceRange[0].high.units, "microgram per deciliter")
        self.assertEqual(inst.referenceRange[0].high.value, 5.0)
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.valueQuantity.code, "ug/dL")
        self.assertEqual(inst.valueQuantity.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.valueQuantity.units, "microgram per deciliter")
        self.assertEqual(inst.valueQuantity.value, 65)
    
    def testObservation4(self):
        inst = self.instantiate_from("obs-uslab-example2.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation4(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation4(inst2)
    
    def implObservation4(self, inst):
        self.assertEqual(inst.appliesDateTime.date, FHIRDate("2014-12-05").date)
        self.assertEqual(inst.appliesDateTime.as_json(), "2014-12-05")
        self.assertEqual(inst.code.coding[0].code, "43305-2")
        self.assertEqual(inst.code.coding[0].display, "Neisseria gonorrhoeae rRNA [Presence] in Unspecified specimen by Probe and target amplification method")
        self.assertEqual(inst.code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.code.coding[1].code, "GCPCR")
        self.assertEqual(inst.code.coding[1].display, "APTIMA GC")
        self.assertEqual(inst.code.coding[1].system, "urn:oid:2.16.840.1.113883.3.72.5.24")
        self.assertEqual(inst.code.text, "Neisseria gonorrhoeae by PCR")
        self.assertEqual(inst.extension[0].url, "http://hl7.org/fhir/StructureDefinition/uslabobservationkind")
        self.assertEqual(inst.extension[0].valueCode, "result")
        self.assertEqual(inst.id, "uslab-example2")
        self.assertEqual(inst.identifier[0].system, "http://lis.acmelabs.org/identifiers/labtestresult")
        self.assertEqual(inst.identifier[0].type.text, "lab test result ID")
        self.assertEqual(inst.identifier[0].use, "official")
        self.assertEqual(inst.identifier[0].value, "2345")
        self.assertEqual(inst.interpretation.coding[0].code, "A")
        self.assertEqual(inst.interpretation.coding[0].system, "http://hl7.org/fhir/v2/0078")
        self.assertEqual(inst.issued.date, FHIRDate("2014-12-06T15:42:15-08:00").date)
        self.assertEqual(inst.issued.as_json(), "2014-12-06T15:42:15-08:00")
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.valueCodeableConcept.coding[0].code, "10828004")
        self.assertEqual(inst.valueCodeableConcept.coding[0].display, "Positive")
        self.assertEqual(inst.valueCodeableConcept.coding[0].system, "http://snomed.info/sct")
        self.assertEqual(inst.valueCodeableConcept.coding[1].code, "POS")
        self.assertEqual(inst.valueCodeableConcept.coding[1].display, "Positive")
        self.assertEqual(inst.valueCodeableConcept.coding[1].system, "urn:oid:2.16.840.1.113883.3.72.5.24")
        self.assertEqual(inst.valueCodeableConcept.text, "Positive for Neisseira gonorrheoea rRNA")
    
    def testObservation5(self):
        inst = self.instantiate_from("obs-uslab-example3.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation5(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation5(inst2)
    
    def implObservation5(self, inst):
        self.assertEqual(inst.appliesDateTime.date, FHIRDate("2014-12-05").date)
        self.assertEqual(inst.appliesDateTime.as_json(), "2014-12-05")
        self.assertEqual(inst.code.coding[0].code, "6463-4")
        self.assertEqual(inst.code.coding[0].display, "Bacteria identified in Unspecified specimen by Culture")
        self.assertEqual(inst.code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.code.coding[1].code, "MSCBACT")
        self.assertEqual(inst.code.coding[1].display, "Misc Culture")
        self.assertEqual(inst.code.coding[1].system, "urn:oid:2.16.840.1.113883.3.72.5.24")
        self.assertEqual(inst.code.text, "Miscellaneous Culture")
        self.assertEqual(inst.extension[0].url, "http://hl7.org/fhir/StructureDefinition/uslabobservationkind")
        self.assertEqual(inst.extension[0].valueCode, "result")
        self.assertEqual(inst.id, "uslab-example3")
        self.assertEqual(inst.identifier[0].system, "http://lis.acmelabs.org/identifiers/labtestresult")
        self.assertEqual(inst.identifier[0].type.text, "lab test result ID")
        self.assertEqual(inst.identifier[0].use, "official")
        self.assertEqual(inst.identifier[0].value, "3456")
        self.assertEqual(inst.interpretation.coding[0].code, "A")
        self.assertEqual(inst.interpretation.coding[0].system, "http://hl7.org/fhir/v2/0078")
        self.assertEqual(inst.issued.date, FHIRDate("2014-12-06T15:42:15-08:00").date)
        self.assertEqual(inst.issued.as_json(), "2014-12-06T15:42:15-08:00")
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.valueString, "Moderate Growth of gram-positive cocci bacteria")
    
    def testObservation6(self):
        inst = self.instantiate_from("obs-uslab-example4.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation6(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation6(inst2)
    
    def implObservation6(self, inst):
        self.assertEqual(inst.appliesDateTime.date, FHIRDate("2014-12-05").date)
        self.assertEqual(inst.appliesDateTime.as_json(), "2014-12-05")
        self.assertEqual(inst.code.coding[0].code, "6812-2")
        self.assertEqual(inst.code.coding[0].display, "Dengue virus IgM Ab [Titer] in Serum")
        self.assertEqual(inst.code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.code.coding[1].code, "DGIGM")
        self.assertEqual(inst.code.coding[1].display, "Dengue IgM QN")
        self.assertEqual(inst.code.coding[1].system, "urn:oid:2.16.840.1.113883.3.72.5.24")
        self.assertEqual(inst.code.text, "Dengue IgM Titer")
        self.assertEqual(inst.extension[0].url, "http://hl7.org/fhir/StructureDefinition/uslabobservationkind")
        self.assertEqual(inst.extension[0].valueCode, "result")
        self.assertEqual(inst.id, "uslab-example4")
        self.assertEqual(inst.identifier[0].system, "http://lis.acmelabs.org/identifiers/labtestresult")
        self.assertEqual(inst.identifier[0].type.text, "lab test result ID")
        self.assertEqual(inst.identifier[0].use, "official")
        self.assertEqual(inst.identifier[0].value, "4567")
        self.assertEqual(inst.interpretation.coding[0].code, "H")
        self.assertEqual(inst.interpretation.coding[0].system, "http://hl7.org/fhir/v2/0078")
        self.assertEqual(inst.issued.date, FHIRDate("2014-12-06T15:42:15-08:00").date)
        self.assertEqual(inst.issued.as_json(), "2014-12-06T15:42:15-08:00")
        self.assertEqual(inst.referenceRange[0].text, "<1:64")
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.valueRatio.denominator.value, 256)
        self.assertEqual(inst.valueRatio.numerator.value, 1)
    
    def testObservation7(self):
        inst = self.instantiate_from("obs-uslab-example5.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation7(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation7(inst2)
    
    def implObservation7(self, inst):
        self.assertEqual(inst.appliesDateTime.date, FHIRDate("2014-12-05").date)
        self.assertEqual(inst.appliesDateTime.as_json(), "2014-12-05")
        self.assertEqual(inst.code.coding[0].code, "5671-3")
        self.assertEqual(inst.code.coding[0].display, "Lead [Mass/volume] in Blood")
        self.assertEqual(inst.code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.code.coding[1].code, "BLDLD")
        self.assertEqual(inst.code.coding[1].display, "Blood Lead")
        self.assertEqual(inst.code.coding[1].system, "urn:oid:2.16.840.1.113883.3.72.5.24")
        self.assertEqual(inst.code.text, "Blood Lead")
        self.assertEqual(inst.extension[0].url, "http://hl7.org/fhir/StructureDefinition/uslabobservationkind")
        self.assertEqual(inst.extension[0].valueCode, "result")
        self.assertEqual(inst.id, "uslab-example5")
        self.assertEqual(inst.identifier[0].system, "http://lis.acmelabs.org/identifiers/labtestresult")
        self.assertEqual(inst.identifier[0].type.text, "lab test result ID")
        self.assertEqual(inst.identifier[0].use, "official")
        self.assertEqual(inst.identifier[0].value, "1234")
        self.assertEqual(inst.interpretation.coding[0].code, "H")
        self.assertEqual(inst.interpretation.coding[0].system, "http://hl7.org/fhir/v2/0078")
        self.assertEqual(inst.issued.date, FHIRDate("2014-12-06T15:42:15-08:00").date)
        self.assertEqual(inst.issued.as_json(), "2014-12-06T15:42:15-08:00")
        self.assertEqual(inst.referenceRange[0].high.code, "ug/dL")
        self.assertEqual(inst.referenceRange[0].high.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.referenceRange[0].high.units, "microgram per deciliter")
        self.assertEqual(inst.referenceRange[0].high.value, 5.0)
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.valueQuantity.code, "ug/dL")
        self.assertEqual(inst.valueQuantity.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.valueQuantity.units, "microgram per deciliter")
        self.assertEqual(inst.valueQuantity.value, 65)
    
    def testObservation8(self):
        inst = self.instantiate_from("obs-uslab-example6.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation8(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation8(inst2)
    
    def implObservation8(self, inst):
        self.assertEqual(inst.appliesDateTime.date, FHIRDate("2014-12-05").date)
        self.assertEqual(inst.appliesDateTime.as_json(), "2014-12-05")
        self.assertEqual(inst.code.coding[0].code, "64017-7")
        self.assertEqual(inst.code.coding[0].display, "Chlamydia trachomatis and Neisseria gonorrhoeae rRNA panel - Unspecified specimen by Probe and target amplification method")
        self.assertEqual(inst.code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.code.coding[1].code, "GCCTPCR")
        self.assertEqual(inst.code.coding[1].display, "GC CT PCR")
        self.assertEqual(inst.code.coding[1].system, "urn:oid:2.16.840.1.113883.3.72.5.24")
        self.assertEqual(inst.code.text, "GC CT RNA Panel")
        self.assertEqual(inst.extension[0].url, "http://hl7.org/fhir/StructureDefinition/uslabobservationkind")
        self.assertEqual(inst.extension[0].valueCode, "result")
        self.assertEqual(inst.id, "uslab-example6")
        self.assertEqual(inst.identifier[0].system, "http://lis.acmelabs.org/identifiers/labtestresult")
        self.assertEqual(inst.identifier[0].type.text, "lab test result ID")
        self.assertEqual(inst.identifier[0].use, "official")
        self.assertEqual(inst.identifier[0].value, "8901")
        self.assertEqual(inst.issued.date, FHIRDate("2014-12-06T15:42:15-08:00").date)
        self.assertEqual(inst.issued.as_json(), "2014-12-06T15:42:15-08:00")
        self.assertEqual(inst.related[0].type, "has-component")
        self.assertEqual(inst.related[1].type, "has-component")
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
    
    def testObservation9(self):
        inst = self.instantiate_from("obs-uslab-example7.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation9(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation9(inst2)
    
    def implObservation9(self, inst):
        self.assertEqual(inst.appliesDateTime.date, FHIRDate("2014-12-05").date)
        self.assertEqual(inst.appliesDateTime.as_json(), "2014-12-05")
        self.assertEqual(inst.code.coding[0].code, "43304-5")
        self.assertEqual(inst.code.coding[0].display, "Chlamydia trachomatis rRNA [Presence] in Unspecified specimen by Probe and target amplification method")
        self.assertEqual(inst.code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.code.coding[1].code, "CTPCR")
        self.assertEqual(inst.code.coding[1].display, "APTIMA CT")
        self.assertEqual(inst.code.coding[1].system, "urn:oid:2.16.840.1.113883.3.72.5.24")
        self.assertEqual(inst.code.text, "Chlamydia trachomatis by PCR")
        self.assertEqual(inst.extension[0].url, "http://hl7.org/fhir/StructureDefinition/uslabobservationkind")
        self.assertEqual(inst.extension[0].valueCode, "result")
        self.assertEqual(inst.id, "uslab-example7")
        self.assertEqual(inst.identifier[0].system, "http://lis.acmelabs.org/identifiers/labtestresult")
        self.assertEqual(inst.identifier[0].type.text, "lab test result ID")
        self.assertEqual(inst.identifier[0].use, "official")
        self.assertEqual(inst.identifier[0].value, "9012")
        self.assertEqual(inst.interpretation.coding[0].code, "N")
        self.assertEqual(inst.interpretation.coding[0].system, "http://hl7.org/fhir/v2/0078")
        self.assertEqual(inst.issued.date, FHIRDate("2014-12-06T15:42:15-08:00").date)
        self.assertEqual(inst.issued.as_json(), "2014-12-06T15:42:15-08:00")
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.valueCodeableConcept.coding[0].code, "260385009")
        self.assertEqual(inst.valueCodeableConcept.coding[0].display, "Negative")
        self.assertEqual(inst.valueCodeableConcept.coding[0].system, "http://snomed.info/sct")
        self.assertEqual(inst.valueCodeableConcept.coding[1].code, "NEG")
        self.assertEqual(inst.valueCodeableConcept.coding[1].display, "Negative")
        self.assertEqual(inst.valueCodeableConcept.coding[1].system, "urn:oid:2.16.840.1.113883.3.72.5.24")
        self.assertEqual(inst.valueCodeableConcept.text, "Negative for Chlamydia Trachomatis rRNA")
    
    def testObservation10(self):
        inst = self.instantiate_from("obs-uslab-example8.json")
        self.assertIsNotNone(inst, "Must have instantiated a Observation instance")
        self.implObservation10(inst)
        
        js = inst.as_json()
        self.assertEqual("Observation", js["resourceType"])
        inst2 = observation.Observation(js)
        self.implObservation10(inst2)
    
    def implObservation10(self, inst):
        self.assertEqual(inst.appliesDateTime.date, FHIRDate("2014-12-03").date)
        self.assertEqual(inst.appliesDateTime.as_json(), "2014-12-03")
        self.assertEqual(inst.code.coding[0].code, "5821-4")
        self.assertEqual(inst.code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.code.coding[1].code, "WBCFLD")
        self.assertEqual(inst.code.coding[1].display, "White blood cells (WBC) in urine ")
        self.assertEqual(inst.code.coding[1].system, "urn:oid:2.16.840.1.113883.3.72.5.24")
        self.assertEqual(inst.code.text, "White blood cells (WBC) in urine")
        self.assertEqual(inst.extension[0].url, "http://hl7.org/fhir/StructureDefinition/uslabobservationkind")
        self.assertEqual(inst.extension[0].valueCode, "result")
        self.assertEqual(inst.id, "uslab-example8")
        self.assertEqual(inst.identifier[0].system, "http://lis.acmelabs.org/identifiers/labtestresult")
        self.assertEqual(inst.identifier[0].type.text, "lab test result ID")
        self.assertEqual(inst.identifier[0].use, "official")
        self.assertEqual(inst.identifier[0].value, "9099")
        self.assertEqual(inst.interpretation.coding[0].code, "H")
        self.assertEqual(inst.interpretation.coding[0].system, "http://hl7.org/fhir/v2/0078")
        self.assertEqual(inst.issued.date, FHIRDate("2014-12-04T15:42:15-08:00").date)
        self.assertEqual(inst.issued.as_json(), "2014-12-04T15:42:15-08:00")
        self.assertEqual(inst.referenceRange[0].high.code, "/[HPF]")
        self.assertEqual(inst.referenceRange[0].high.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.referenceRange[0].high.units, "WBC/HPF")
        self.assertEqual(inst.referenceRange[0].high.value, 5)
        self.assertEqual(inst.referenceRange[0].low.code, "/[HPF]")
        self.assertEqual(inst.referenceRange[0].low.system, "http://unitsofmeasure.org")
        self.assertEqual(inst.referenceRange[0].low.units, "WBC/HPF")
        self.assertEqual(inst.referenceRange[0].low.value, 0)
        self.assertEqual(inst.status, "final")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.valueRange.high.units, "WBC/HPF")
        self.assertEqual(inst.valueRange.high.value, 20)
        self.assertEqual(inst.valueRange.low.units, "WBC/HPF")
        self.assertEqual(inst.valueRange.low.value, 10)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 0.5.0.5149 (http://hl7.org/fhir/StructureDefinition/MedicationPrescription) on 2015-07-06.
#  2015, SMART Health IT.


from . import codeableconcept
from . import domainresource
from . import duration
from . import fhirdate
from . import fhirelement
from . import fhirreference
from . import identifier
from . import period
from . import quantity
from . import range
from . import ratio
from . import timing


class MedicationPrescription(domainresource.DomainResource):
    """ Prescription of medication to for patient.
    
    An order for both supply of the medication and the instructions for
    administration of the medicine to a patient.
    """
    
    resource_name = "MedicationPrescription"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.dateWritten = None
        """ When prescription was authorized.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.dispense = None
        """ Medication supply authorization.
        Type `MedicationPrescriptionDispense` (represented as `dict` in JSON). """
        
        self.dosageInstruction = None
        """ How medication should be taken.
        List of `MedicationPrescriptionDosageInstruction` items (represented as `dict` in JSON). """
        
        self.encounter = None
        """ Created during encounter / admission / stay.
        Type `FHIRReference` referencing `Encounter` (represented as `dict` in JSON). """
        
        self.identifier = None
        """ External identifier.
        List of `Identifier` items (represented as `dict` in JSON). """
        
        self.medication = None
        """ Medication to be taken.
        Type `FHIRReference` referencing `Medication` (represented as `dict` in JSON). """
        
        self.note = None
        """ Information about the prescription.
        Type `str`. """
        
        self.patient = None
        """ Who prescription is for.
        Type `FHIRReference` referencing `Patient` (represented as `dict` in JSON). """
        
        self.prescriber = None
        """ Who ordered the medication(s).
        Type `FHIRReference` referencing `Practitioner` (represented as `dict` in JSON). """
        
        self.reasonCodeableConcept = None
        """ Reason or indication for writing the prescription.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.reasonReference = None
        """ Reason or indication for writing the prescription.
        Type `FHIRReference` referencing `Condition` (represented as `dict` in JSON). """
        
        self.status = None
        """ active | on-hold | completed | entered-in-error | stopped |
        superceded | draft.
        Type `str`. """
        
        self.substitution = None
        """ Any restrictions on medication substitution?.
        Type `MedicationPrescriptionSubstitution` (represented as `dict` in JSON). """
        
        super(MedicationPrescription, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(MedicationPrescription, self).elementProperties()
        js.extend([
            ("dateWritten", "dateWritten", fhirdate.FHIRDate, False),
            ("dispense", "dispense", MedicationPrescriptionDispense, False),
            ("dosageInstruction", "dosageInstruction", MedicationPrescriptionDosageInstruction, True),
            ("encounter", "encounter", fhirreference.FHIRReference, False),
            ("identifier", "identifier", identifier.Identifier, True),
            ("medication", "medication", fhirreference.FHIRReference, False),
            ("note", "note", str, False),
            ("patient", "patient", fhirreference.FHIRReference, False),
            ("prescriber", "prescriber", fhirreference.FHIRReference, False),
            ("reasonCodeableConcept", "reasonCodeableConcept", codeableconcept.CodeableConcept, False),
            ("reasonReference", "reasonReference", fhirreference.FHIRReference, False),
            ("status", "status", str, False),
            ("substitution", "substitution", MedicationPrescriptionSubstitution, False),
        ])
        return js


class MedicationPrescriptionDispense(fhirelement.FHIRElement):
    """ Medication supply authorization.
    
    Deals with details of the dispense part of the order.
    """
    
    resource_name = "MedicationPrescriptionDispense"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.expectedSupplyDuration = None
        """ Days supply per dispense.
        Type `Duration` (represented as `dict` in JSON). """
        
        self.medication = None
        """ Product to be supplied.
        Type `FHIRReference` referencing `Medication` (represented as `dict` in JSON). """
        
        self.numberOfRepeatsAllowed = None
        """ # of refills authorized.
        Type `int`. """
        
        self.quantity = None
        """ Amount of medication to supply per dispense.
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.validityPeriod = None
        """ Time period supply is authorized for.
        Type `Period` (represented as `dict` in JSON). """
        
        super(MedicationPrescriptionDispense, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(MedicationPrescriptionDispense, self).elementProperties()
        js.extend([
            ("expectedSupplyDuration", "expectedSupplyDuration", duration.Duration, False),
            ("medication", "medication", fhirreference.FHIRReference, False),
            ("numberOfRepeatsAllowed", "numberOfRepeatsAllowed", int, False),
            ("quantity", "quantity", quantity.Quantity, False),
            ("validityPeriod", "validityPeriod", period.Period, False),
        ])
        return js


class MedicationPrescriptionDosageInstruction(fhirelement.FHIRElement):
    """ How medication should be taken.
    
    Indicates how the medication is to be used by the patient.
    """
    
    resource_name = "MedicationPrescriptionDosageInstruction"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.additionalInstructions = None
        """ Supplemental instructions - e.g. "with meals".
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.asNeededBoolean = None
        """ Take "as needed" f(or x).
        Type `bool`. """
        
        self.asNeededCodeableConcept = None
        """ Take "as needed" f(or x).
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.doseQuantity = None
        """ Amount of medication per dose.
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.doseRange = None
        """ Amount of medication per dose.
        Type `Range` (represented as `dict` in JSON). """
        
        self.maxDosePerPeriod = None
        """ Upper limit on medication per unit of time.
        Type `Ratio` (represented as `dict` in JSON). """
        
        self.method = None
        """ Technique for administering medication.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.rate = None
        """ Amount of medication per unit of time.
        Type `Ratio` (represented as `dict` in JSON). """
        
        self.route = None
        """ How drug should enter body.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.scheduledDateTime = None
        """ When medication should be administered.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.scheduledPeriod = None
        """ When medication should be administered.
        Type `Period` (represented as `dict` in JSON). """
        
        self.scheduledTiming = None
        """ When medication should be administered.
        Type `Timing` (represented as `dict` in JSON). """
        
        self.site = None
        """ Body site to administer to.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.text = None
        """ Dosage instructions expressed as text.
        Type `str`. """
        
        super(MedicationPrescriptionDosageInstruction, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(MedicationPrescriptionDosageInstruction, self).elementProperties()
        js.extend([
            ("additionalInstructions", "additionalInstructions", codeableconcept.CodeableConcept, False),
            ("asNeededBoolean", "asNeededBoolean", bool, False),
            ("asNeededCodeableConcept", "asNeededCodeableConcept", codeableconcept.CodeableConcept, False),
            ("doseQuantity", "doseQuantity", quantity.Quantity, False),
            ("doseRange", "doseRange", range.Range, False),
            ("maxDosePerPeriod", "maxDosePerPeriod", ratio.Ratio, False),
            ("method", "method", codeableconcept.CodeableConcept, False),
            ("rate", "rate", ratio.Ratio, False),
            ("route", "route", codeableconcept.CodeableConcept, False),
            ("scheduledDateTime", "scheduledDateTime", fhirdate.FHIRDate, False),
            ("scheduledPeriod", "scheduledPeriod", period.Period, False),
            ("scheduledTiming", "scheduledTiming", timing.Timing, False),
            ("site", "site", codeableconcept.CodeableConcept, False),
            ("text", "text", str, False),
        ])
        return js


class MedicationPrescriptionSubstitution(fhirelement.FHIRElement):
    """ Any restrictions on medication substitution?.
    
    Indicates whether or not substitution can or should be part of the
    dispense. In some cases substitution must happen, in other cases
    substitution must not happen, and in others it does not matter. This block
    explains the prescriber's intent. If nothing is specified substitution may
    be done.
    """
    
    resource_name = "MedicationPrescriptionSubstitution"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.reason = None
        """ Why should substitution (not) be made.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.type = None
        """ generic | formulary +.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        super(MedicationPrescriptionSubstitution, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(MedicationPrescriptionSubstitution, self).elementProperties()
        js.extend([
            ("reason", "reason", codeableconcept.CodeableConcept, False),
            ("type", "type", codeableconcept.CodeableConcept, False),
        ])
        return js




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 0.5.0.5149 on 2015-07-06.
#  2015, SMART Health IT.


import os
import io
import unittest
import json
from . import specimen
from .fhirdate import FHIRDate


class SpecimenTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("Specimen", js["resourceType"])
        return specimen.Specimen(js)
    
    def testSpecimen1(self):
        inst = self.instantiate_from("spec-uslab-example1.json")
        self.assertIsNotNone(inst, "Must have instantiated a Specimen instance")
        self.implSpecimen1(inst)
        
        js = inst.as_json()
        self.assertEqual("Specimen", js["resourceType"])
        inst2 = specimen.Specimen(js)
        self.implSpecimen1(inst2)
    
    def implSpecimen1(self, inst):
        self.assertEqual(inst.accessionIdentifier.system, "http://lis.acmelabs.org/identifiers/accession")
        self.assertEqual(inst.accessionIdentifier.use, "official")
        self.assertEqual(inst.accessionIdentifier.value, "21041205000001")
        self.assertEqual(inst.collection.bodySiteCodeableConcept.coding[0].code, " 53120007")
        self.assertEqual(inst.collection.bodySiteCodeableConcept.coding[0].display, "Arm")
        self.assertEqual(inst.collection.bodySiteCodeableConcept.coding[0].system, "http://snomed.info/sct")
        self.assertEqual(inst.collection.bodySiteCodeableConcept.coding[1].code, "ARM")
        self.assertEqual(inst.collection.bodySiteCodeableConcept.coding[1].display, "Arm")
        self.assertEqual(inst.collection.bodySiteCodeableConcept.coding[1].system, "http://ehr.goodhealthclinic.org")
        self.assertEqual(inst.collection.bodySiteCodeableConcept.text, "Drawn from Arm")
        self.assertEqual(inst.collection.collectedDateTime.date, FHIRDate("2014-12-05").date)
        self.assertEqual(inst.collection.collectedDateTime.as_json(), "2014-12-05")
        self.assertEqual(inst.id, "uslab-example1")
        self.assertEqual(inst.identifier[0].system, "http://ehr.goodhealthclinic.org/identifiers/specimen")
        self.assertEqual(inst.identifier[0].use, "official")
        self.assertEqual(inst.identifier[0].value, "SID123")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.type.coding[0].code, "122555007")
        self.assertEqual(inst.type.coding[0].display, "Venous blood specimen")
        self.assertEqual(inst.type.coding[0].system, "http://snomed.info/sct")
        self.assertEqual(inst.type.coding[1].code, "BLD")
        self.assertEqual(inst.type.coding[1].display, "Blood")
        self.assertEqual(inst.type.coding[1].system, "http://ehr.goodhealthclinic.org")
        self.assertEqual(inst.type.text, "Blood sample")
    
    def testSpecimen2(self):
        inst = self.instantiate_from("spec-uslab-example2.json")
        self.assertIsNotNone(inst, "Must have instantiated a Specimen instance")
        self.implSpecimen2(inst)
        
        js = inst.as_json()
        self.assertEqual("Specimen", js["resourceType"])
        inst2 = specimen.Specimen(js)
        self.implSpecimen2(inst2)
    
    def implSpecimen2(self, inst):
        self.assertEqual(inst.accessionIdentifier.system, "http://lis.acmelabs.org/identifiers/accession")
        self.assertEqual(inst.accessionIdentifier.use, "official")
        self.assertEqual(inst.accessionIdentifier.value, "21041205000001")
        self.assertEqual(inst.collection.collectedDateTime.date, FHIRDate("2014-12-05").date)
        self.assertEqual(inst.collection.collectedDateTime.as_json(), "2014-12-05")
        self.assertEqual(inst.id, "uslab-example2")
        self.assertEqual(inst.identifier[0].system, "http://ehr.goodhealthclinic.org/identifiers/specimen")
        self.assertEqual(inst.identifier[0].use, "official")
        self.assertEqual(inst.identifier[0].value, "SID456")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.type.coding[0].code, "698276005")
        self.assertEqual(inst.type.coding[0].display, "First stream urine sample")
        self.assertEqual(inst.type.coding[0].system, "http://snomed.info/sct")
        self.assertEqual(inst.type.coding[1].code, "UFV")
        self.assertEqual(inst.type.coding[1].display, "Urine, First Void")
        self.assertEqual(inst.type.coding[1].system, "http://ehr.goodhealthclinic.org")
        self.assertEqual(inst.type.text, "Urine, First Void")
    
    def testSpecimen3(self):
        inst = self.instantiate_from("specimen-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a Specimen instance")
        self.implSpecimen3(inst)
        
        js = inst.as_json()
        self.assertEqual("Specimen", js["resourceType"])
        inst2 = specimen.Specimen(js)
        self.implSpecimen3(inst2)
    
    def implSpecimen3(self, inst):
        self.assertEqual(inst.accessionIdentifier.system, "http://lab.acme.org/specimens/2011")
        self.assertEqual(inst.accessionIdentifier.value, "X352356")
        self.assertEqual(inst.collection.collectedDateTime.date, FHIRDate("2011-05-30T06:15:00Z").date)
        self.assertEqual(inst.collection.collectedDateTime.as_json(), "2011-05-30T06:15:00Z")
        self.assertEqual(inst.collection.method.coding[0].code, "LNV")
        self.assertEqual(inst.collection.method.coding[0].system, "http://hl7.org/fhir/v2/0488")
        self.assertEqual(inst.collection.quantity.units, "mL")
        self.assertEqual(inst.collection.quantity.value, 6)
        self.assertEqual(inst.container[0].capacity.units, "mL")
        self.assertEqual(inst.container[0].capacity.value, 10)
        self.assertEqual(inst.container[0].description, "Green Gel tube")
        self.assertEqual(inst.container[0].identifier[0].value, "48736-15394-75465")
        self.assertEqual(inst.container[0].specimenQuantity.units, "mL")
        self.assertEqual(inst.container[0].specimenQuantity.value, 6)
        self.assertEqual(inst.container[0].type.text, "Vacutainer")
        self.assertEqual(inst.id, "101")
        self.assertEqual(inst.identifier[0].system, "http://ehr.acme.org/identifiers/collections")
        self.assertEqual(inst.identifier[0].value, "23234352356")
        self.assertEqual(inst.receivedTime.date, FHIRDate("2011-03-04T07:03:00Z").date)
        self.assertEqual(inst.receivedTime.as_json(), "2011-03-04T07:03:00Z")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.type.coding[0].code, "122555007")
        self.assertEqual(inst.type.coding[0].display, "Venous blood specimen")
        self.assertEqual(inst.type.coding[0].system, "http://snomed.info/sct")
    
    def testSpecimen4(self):
        inst = self.instantiate_from("specimen-qicore-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a Specimen instance")
        self.implSpecimen4(inst)
        
        js = inst.as_json()
        self.assertEqual("Specimen", js["resourceType"])
        inst2 = specimen.Specimen(js)
        self.implSpecimen4(inst2)
    
    def implSpecimen4(self, inst):
        self.assertEqual(inst.collection.collectedDateTime.date, FHIRDate("2011-03-06T06:15:00Z").date)
        self.assertEqual(inst.collection.collectedDateTime.as_json(), "2011-03-06T06:15:00Z")
        self.assertEqual(inst.collection.extension[0].url, "http://hl7.org/fhir/StructureDefinition/specimen-collectionPriority")
        self.assertEqual(inst.collection.extension[0].valueCodeableConcept.coding[0].code, "5")
        self.assertEqual(inst.collection.extension[0].valueCodeableConcept.coding[0].display, "ROUTINE")
        self.assertEqual(inst.collection.extension[0].valueCodeableConcept.coding[0].system, "http://hl7.org/fhir/vs/specimen-collection-priority")
        self.assertEqual(inst.collection.extension[1].url, "http://hl7.org/fhir/StructureDefinition/specimen-specialHandling")
        self.assertEqual(inst.collection.extension[1].valueCodeableConcept.coding[0].code, "NOPERSISTP")
        self.assertEqual(inst.collection.extension[1].valueCodeableConcept.coding[0].display, "no collection beyond purpose of use")
        self.assertEqual(inst.collection.extension[1].valueCodeableConcept.coding[0].system, "http://hl7.org/fhir/v3/ActCode")
        self.assertEqual(inst.collection.quantity.extension[0].url, "http://hl7.org/fhir/StructureDefinition/specimen-isDryWeight")
        self.assertFalse(inst.collection.quantity.extension[0].valueBoolean)
        self.assertEqual(inst.collection.quantity.units, "mL")
        self.assertEqual(inst.collection.quantity.value, 6)
        self.assertEqual(inst.container[0].capacity.units, "mL")
        self.assertEqual(inst.container[0].capacity.value, 10)
        self.assertEqual(inst.container[0].extension[0].url, "http://hl7.org/fhir/StructureDefinition/specimen-sequenceNumber")
        self.assertEqual(inst.container[0].extension[0].valueInteger, 1)
        self.assertEqual(inst.container[0].type.coding[0].code, "434746001")
        self.assertEqual(inst.container[0].type.coding[0].display, "Specimen vial")
        self.assertEqual(inst.container[0].type.coding[0].system, "http://snomed.info/sct")
        self.assertEqual(inst.id, "qicore")
        self.assertEqual(inst.receivedTime.date, FHIRDate("2011-03-04T07:03:00Z").date)
        self.assertEqual(inst.receivedTime.as_json(), "2011-03-04T07:03:00Z")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.treatment[0].description, "Treated with anticoagulants.")
        self.assertEqual(inst.treatment[0].extension[0].url, "http://hl7.org/fhir/StructureDefinition/specimen-treatmentTime")
        self.assertEqual(inst.treatment[0].extension[0].valuePeriod.end.date, FHIRDate("2011-03-04T07:03:00Z").date)
        self.assertEqual(inst.treatment[0].extension[0].valuePeriod.end.as_json(), "2011-03-04T07:03:00Z")
        self.assertEqual(inst.treatment[0].extension[0].valuePeriod.start.date, FHIRDate("2011-03-04T07:03:00Z").date)
        self.assertEqual(inst.treatment[0].extension[0].valuePeriod.start.as_json(), "2011-03-04T07:03:00Z")
        self.assertEqual(inst.type.coding[0].code, "122555007")
        self.assertEqual(inst.type.coding[0].display, "Venous blood specimen")
        self.assertEqual(inst.type.coding[0].system, "http://snomed.info/sct")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Base class for all FHIR elements.

import logging


class FHIRElement(object):
    """ Base class for all FHIR elements.
    """
    
    def __init__(self, jsondict=None):
        self.id = None
        """ Logical id of this artefact. """
        
        self.contained = None
        """ Contained resources. """
        
        self._resolved = None
        """ Dictionary of resolved resources. """
        
        self._owner = None
        """ Points to the parent resource, if there is one. """
        
        self.extension = None
        self.modifierExtension = None
        
        if jsondict is not None:
            self.update_with_json(jsondict)
    
    
    # MARK: Instantiation from JSON
    
    @classmethod
    def with_json(cls, jsonobj):
        """ Initialize an element from a JSON dictionary or array.
        
        :param jsonobj: A dict or list of dicts to instantiate from
        :returns: An instance or a list of instances created from JSON data
        """
        if isinstance(jsonobj, dict):
            return cls(jsonobj)
        
        arr = []
        for jsondict in jsonobj:
            arr.append(cls(jsondict))
        return arr
    
    @classmethod
    def with_json_and_owner(cls, jsonobj, owner):
        """ Instantiates by forwarding to `with_json()`, then remembers the
        "owner" of the instantiated elements. The "owner" is the resource
        containing the receiver and is used to resolve contained resources.
        
        :param dict jsonobj: Decoded JSON dictionary
        :param FHIRElement owner: The owning parent
        :returns: An instance or a list of instances created from JSON data
        """
        instance = cls.with_json(jsonobj)
        if isinstance(instance, list):
            for inst in instance:
                inst._owner = owner
        else:
            instance._owner = owner
        
        return instance
    
    
    # MARK: (De)Serialization
    
    def elementProperties(self):
        """ Returns a list of tuples, one tuple for each property that should
        be serialized, as: ("name", "json_name", type, is_list)
        """
        from . import extension
        return [
            ("id", "id", str, False),
            ("extension", "extension", extension.Extension, True),
            ("modifierExtension", "modifierExtension", extension.Extension, True),
        ]
    
    def update_with_json(self, jsondict):
        """ Update the receiver with data in a JSON dictionary.
        """
        if jsondict is None:
            return
        if not isinstance(jsondict, dict):
            logging.warning("Non-dict type {} fed to `update_with_json` on {}"
                .format(type(jsondict), type(self)))
            return
        
        # loop all registered properties and instantiate
        for prop, name, typ, is_list in self.elementProperties():
            if not name in jsondict:
                continue
            
            if hasattr(typ, 'with_json_and_owner'):
                setattr(self, prop, typ.with_json_and_owner(jsondict[name], self))
            else:
                setattr(self, prop, jsondict[name])
        
        # extract contained resources
        if 'contained' in jsondict:
            from . import fhircontainedresource
            self.contained = self.contained or {}
            for js in jsondict['contained']:         # "contained" should be an array
                res = fhircontainedresource.FHIRContainedResource(jsondict=js, owner=self)
                if res.id:
                    self.contained[res.id] = res
                else:
                    logging.warning("Contained resource {} does not have an id, ignoring".format(res))
    
    def as_json(self):
        """ Serializes to JSON by inspecting `elementProperties()` and creating
        a JSON dictionary of all registered properties.
        """
        js = {}
        
        # JSONify all registered properties
        for prop, name, typ, is_list in self.elementProperties():
            val = getattr(self, prop)
            if val is None:
                continue
            if is_list:
                js[name] = [v.as_json() if hasattr(v, 'as_json') else v for v in val]
            else:
                js[name] = val.as_json() if hasattr(val, 'as_json') else val
        
        # handle contained resources
        if self.contained is not None and len(self.contained) > 0:
            # TODO: look at resolved (and newly added?) contained resources
            js['contained'] = [v.as_json() for k, v in self.contained.items()]
        
        return js
    
    
    # MARK: Handling References
    
    def containedReference(self, refid):
        """ Looks for the contained reference with the given id.
        
        :returns: An instance of FHIRContainedResource, if it was found
        """
        if self.contained and refid in self.contained:
            return self.contained[refid]
        return self._owner.containedReference(refid) if self._owner is not None else None
    
    def resolvedReference(self, refid):
        """ Returns the resolved reference with the given id, if it has been
        resolved already.
        """
        if self._resolved and refid in self._resolved:
            return self._resolved[refid]
        return self._owner.resolvedReference(refid) if self._owner is not None else None
    
    def didResolveReference(self, refid, resolved):
        """ Called by FHIRResource when it resolves a reference. Stores the
        resolved reference into the `_resolved` dictionary of the topmost
        owner.
        """
        if self._resolved is not None:
            self._resolved[refid] = resolved
        else:
            self._resolved = {refid: resolved}




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 0.5.0.5149 (http://hl7.org/fhir/StructureDefinition/Reference) on 2015-07-06.
#  2015, SMART Health IT.


from . import fhirelement


class Reference(fhirelement.FHIRElement):
    """ A reference from one resource to another.
    """
    
    resource_name = "Reference"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.display = None
        """ Text alternative for the resource.
        Type `str`. """
        
        self.reference = None
        """ Relative, internal or absolute URL reference.
        Type `str`. """
        
        super(Reference, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(Reference, self).elementProperties()
        js.extend([
            ("display", "display", str, False),
            ("reference", "reference", str, False),
        ])
        return js




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 0.5.0.5149 (http://hl7.org/fhir/StructureDefinition/BackboneElement) on 2015-07-06.
#  2015, SMART Health IT.


from . import fhirelement


class BackboneElement(fhirelement.FHIRElement):
    """ Base for elements defined inside a resource.
    
    Base definition for all elements that are defined inside a resource - but
    not those in a data type.
    """
    
    resource_name = "BackboneElement"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        super(BackboneElement, self).__init__(jsondict)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 0.5.0.5149 (http://hl7.org/fhir/StructureDefinition/MedicationDispense) on 2015-07-06.
#  2015, SMART Health IT.


from . import codeableconcept
from . import domainresource
from . import fhirdate
from . import fhirelement
from . import fhirreference
from . import identifier
from . import period
from . import quantity
from . import range
from . import ratio
from . import timing


class MedicationDispense(domainresource.DomainResource):
    """ Dispensing a medication to a named patient.
    
    Dispensing a medication to a named patient.  This includes a description of
    the supply provided and the instructions for administering the medication.
    """
    
    resource_name = "MedicationDispense"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.authorizingPrescription = None
        """ Medication order that authorizes the dispense.
        List of `FHIRReference` items referencing `MedicationPrescription` (represented as `dict` in JSON). """
        
        self.daysSupply = None
        """ Days Supply.
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.destination = None
        """ Where the medication was sent.
        Type `FHIRReference` referencing `Location` (represented as `dict` in JSON). """
        
        self.dispenser = None
        """ Practitioner responsible for dispensing medication.
        Type `FHIRReference` referencing `Practitioner` (represented as `dict` in JSON). """
        
        self.dosageInstruction = None
        """ Medicine administration instructions to the patient/carer.
        List of `MedicationDispenseDosageInstruction` items (represented as `dict` in JSON). """
        
        self.identifier = None
        """ External identifier.
        Type `Identifier` (represented as `dict` in JSON). """
        
        self.medication = None
        """ What medication was supplied.
        Type `FHIRReference` referencing `Medication` (represented as `dict` in JSON). """
        
        self.note = None
        """ Information about the dispense.
        Type `str`. """
        
        self.patient = None
        """ Who the dispense is for.
        Type `FHIRReference` referencing `Patient` (represented as `dict` in JSON). """
        
        self.quantity = None
        """ Amount dispensed.
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.receiver = None
        """ Who collected the medication.
        List of `FHIRReference` items referencing `Patient, Practitioner` (represented as `dict` in JSON). """
        
        self.status = None
        """ in-progress | on-hold | completed | entered-in-error | stopped.
        Type `str`. """
        
        self.substitution = None
        """ Deals with substitution of one medicine for another.
        Type `MedicationDispenseSubstitution` (represented as `dict` in JSON). """
        
        self.type = None
        """ Trial fill, partial fill, emergency fill, etc..
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.whenHandedOver = None
        """ Handover time.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.whenPrepared = None
        """ Dispense processing time.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        super(MedicationDispense, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(MedicationDispense, self).elementProperties()
        js.extend([
            ("authorizingPrescription", "authorizingPrescription", fhirreference.FHIRReference, True),
            ("daysSupply", "daysSupply", quantity.Quantity, False),
            ("destination", "destination", fhirreference.FHIRReference, False),
            ("dispenser", "dispenser", fhirreference.FHIRReference, False),
            ("dosageInstruction", "dosageInstruction", MedicationDispenseDosageInstruction, True),
            ("identifier", "identifier", identifier.Identifier, False),
            ("medication", "medication", fhirreference.FHIRReference, False),
            ("note", "note", str, False),
            ("patient", "patient", fhirreference.FHIRReference, False),
            ("quantity", "quantity", quantity.Quantity, False),
            ("receiver", "receiver", fhirreference.FHIRReference, True),
            ("status", "status", str, False),
            ("substitution", "substitution", MedicationDispenseSubstitution, False),
            ("type", "type", codeableconcept.CodeableConcept, False),
            ("whenHandedOver", "whenHandedOver", fhirdate.FHIRDate, False),
            ("whenPrepared", "whenPrepared", fhirdate.FHIRDate, False),
        ])
        return js


class MedicationDispenseDosageInstruction(fhirelement.FHIRElement):
    """ Medicine administration instructions to the patient/carer.
    
    Indicates how the medication is to be used by the patient.
    """
    
    resource_name = "MedicationDispenseDosageInstruction"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.additionalInstructions = None
        """ E.g. "Take with food".
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.asNeededBoolean = None
        """ Take "as needed" f(or x).
        Type `bool`. """
        
        self.asNeededCodeableConcept = None
        """ Take "as needed" f(or x).
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.doseQuantity = None
        """ Amount of medication per dose.
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.doseRange = None
        """ Amount of medication per dose.
        Type `Range` (represented as `dict` in JSON). """
        
        self.maxDosePerPeriod = None
        """ Upper limit on medication per unit of time.
        Type `Ratio` (represented as `dict` in JSON). """
        
        self.method = None
        """ Technique for administering medication.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.rate = None
        """ Amount of medication per unit of time.
        Type `Ratio` (represented as `dict` in JSON). """
        
        self.route = None
        """ How drug should enter body.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.scheduleDateTime = None
        """ When medication should be administered.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.schedulePeriod = None
        """ When medication should be administered.
        Type `Period` (represented as `dict` in JSON). """
        
        self.scheduleTiming = None
        """ When medication should be administered.
        Type `Timing` (represented as `dict` in JSON). """
        
        self.site = None
        """ Body site to administer to.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        super(MedicationDispenseDosageInstruction, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(MedicationDispenseDosageInstruction, self).elementProperties()
        js.extend([
            ("additionalInstructions", "additionalInstructions", codeableconcept.CodeableConcept, False),
            ("asNeededBoolean", "asNeededBoolean", bool, False),
            ("asNeededCodeableConcept", "asNeededCodeableConcept", codeableconcept.CodeableConcept, False),
            ("doseQuantity", "doseQuantity", quantity.Quantity, False),
            ("doseRange", "doseRange", range.Range, False),
            ("maxDosePerPeriod", "maxDosePerPeriod", ratio.Ratio, False),
            ("method", "method", codeableconcept.CodeableConcept, False),
            ("rate", "rate", ratio.Ratio, False),
            ("route", "route", codeableconcept.CodeableConcept, False),
            ("scheduleDateTime", "scheduleDateTime", fhirdate.FHIRDate, False),
            ("schedulePeriod", "schedulePeriod", period.Period, False),
            ("scheduleTiming", "scheduleTiming", timing.Timing, False),
            ("site", "site", codeableconcept.CodeableConcept, False),
        ])
        return js


class MedicationDispenseSubstitution(fhirelement.FHIRElement):
    """ Deals with substitution of one medicine for another.
    
    Indicates whether or not substitution was made as part of the dispense.  In
    some cases substitution will be expected but doesn't happen, in other cases
    substitution is not expected but does happen.  This block explains what
    substitition did or did not happen and why.
    """
    
    resource_name = "MedicationDispenseSubstitution"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.reason = None
        """ Why was substitution made.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.responsibleParty = None
        """ Who is responsible for the substitution.
        List of `FHIRReference` items referencing `Practitioner` (represented as `dict` in JSON). """
        
        self.type = None
        """ Type of substitiution.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        super(MedicationDispenseSubstitution, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(MedicationDispenseSubstitution, self).elementProperties()
        js.extend([
            ("reason", "reason", codeableconcept.CodeableConcept, True),
            ("responsibleParty", "responsibleParty", fhirreference.FHIRReference, True),
            ("type", "type", codeableconcept.CodeableConcept, False),
        ])
        return js




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 0.5.0.5149 (http://hl7.org/fhir/StructureDefinition/Encounter) on 2015-07-06.
#  2015, SMART Health IT.


from . import codeableconcept
from . import domainresource
from . import duration
from . import fhirelement
from . import fhirreference
from . import identifier
from . import period


class Encounter(domainresource.DomainResource):
    """ An interaction during which services are provided to the patient.
    
    An interaction between a patient and healthcare provider(s) for the purpose
    of providing healthcare service(s) or assessing the health status of a
    patient.
    """
    
    resource_name = "Encounter"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.episodeOfCare = None
        """ An episode of care that this encounter should be recorded against.
        Type `FHIRReference` referencing `EpisodeOfCare` (represented as `dict` in JSON). """
        
        self.fulfills = None
        """ The appointment that scheduled this encounter.
        Type `FHIRReference` referencing `Appointment` (represented as `dict` in JSON). """
        
        self.hospitalization = None
        """ Details about an admission to a clinic.
        Type `EncounterHospitalization` (represented as `dict` in JSON). """
        
        self.identifier = None
        """ Identifier(s) by which this encounter is known.
        List of `Identifier` items (represented as `dict` in JSON). """
        
        self.incomingReferralRequest = None
        """ Incoming Referral Request.
        List of `FHIRReference` items referencing `ReferralRequest` (represented as `dict` in JSON). """
        
        self.indication = None
        """ Reason the encounter takes place (resource).
        List of `FHIRReference` items referencing `Resource` (represented as `dict` in JSON). """
        
        self.klass = None
        """ inpatient | outpatient | ambulatory | emergency +.
        Type `str`. """
        
        self.length = None
        """ Quantity of time the encounter lasted (less time absent).
        Type `Duration` (represented as `dict` in JSON). """
        
        self.location = None
        """ List of locations the patient has been at.
        List of `EncounterLocation` items (represented as `dict` in JSON). """
        
        self.partOf = None
        """ Another Encounter this encounter is part of.
        Type `FHIRReference` referencing `Encounter` (represented as `dict` in JSON). """
        
        self.participant = None
        """ List of participants involved in the encounter.
        List of `EncounterParticipant` items (represented as `dict` in JSON). """
        
        self.patient = None
        """ The patient present at the encounter.
        Type `FHIRReference` referencing `Patient` (represented as `dict` in JSON). """
        
        self.period = None
        """ The start and end time of the encounter.
        Type `Period` (represented as `dict` in JSON). """
        
        self.priority = None
        """ Indicates the urgency of the encounter.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.reason = None
        """ Reason the encounter takes place (code).
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.serviceProvider = None
        """ The custodian organization of this Encounter record.
        Type `FHIRReference` referencing `Organization` (represented as `dict` in JSON). """
        
        self.status = None
        """ planned | arrived | in-progress | onleave | finished | cancelled.
        Type `str`. """
        
        self.statusHistory = None
        """ List of Encounter statuses.
        List of `EncounterStatusHistory` items (represented as `dict` in JSON). """
        
        self.type = None
        """ Specific type of encounter.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        super(Encounter, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(Encounter, self).elementProperties()
        js.extend([
            ("episodeOfCare", "episodeOfCare", fhirreference.FHIRReference, False),
            ("fulfills", "fulfills", fhirreference.FHIRReference, False),
            ("hospitalization", "hospitalization", EncounterHospitalization, False),
            ("identifier", "identifier", identifier.Identifier, True),
            ("incomingReferralRequest", "incomingReferralRequest", fhirreference.FHIRReference, True),
            ("indication", "indication", fhirreference.FHIRReference, True),
            ("klass", "class", str, False),
            ("length", "length", duration.Duration, False),
            ("location", "location", EncounterLocation, True),
            ("partOf", "partOf", fhirreference.FHIRReference, False),
            ("participant", "participant", EncounterParticipant, True),
            ("patient", "patient", fhirreference.FHIRReference, False),
            ("period", "period", period.Period, False),
            ("priority", "priority", codeableconcept.CodeableConcept, False),
            ("reason", "reason", codeableconcept.CodeableConcept, True),
            ("serviceProvider", "serviceProvider", fhirreference.FHIRReference, False),
            ("status", "status", str, False),
            ("statusHistory", "statusHistory", EncounterStatusHistory, True),
            ("type", "type", codeableconcept.CodeableConcept, True),
        ])
        return js


class EncounterHospitalization(fhirelement.FHIRElement):
    """ Details about an admission to a clinic.
    """
    
    resource_name = "EncounterHospitalization"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.admitSource = None
        """ From where patient was admitted (physician referral, transfer).
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.destination = None
        """ Location to which the patient is discharged.
        Type `FHIRReference` referencing `Location` (represented as `dict` in JSON). """
        
        self.dietPreference = None
        """ Diet preferences reported by the patient.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.dischargeDiagnosis = None
        """ The final diagnosis given a patient before release from the
        hospital after all testing, surgery, and workup are complete.
        Type `FHIRReference` referencing `Resource` (represented as `dict` in JSON). """
        
        self.dischargeDisposition = None
        """ Category or kind of location after discharge.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.origin = None
        """ The location from which the patient came before admission.
        Type `FHIRReference` referencing `Location` (represented as `dict` in JSON). """
        
        self.preAdmissionIdentifier = None
        """ Pre-admission identifier.
        Type `Identifier` (represented as `dict` in JSON). """
        
        self.reAdmission = None
        """ Is this hospitalization a readmission?.
        Type `bool`. """
        
        self.specialArrangement = None
        """ Wheelchair, translator, stretcher, etc.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.specialCourtesy = None
        """ Special courtesies (VIP, board member).
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        super(EncounterHospitalization, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(EncounterHospitalization, self).elementProperties()
        js.extend([
            ("admitSource", "admitSource", codeableconcept.CodeableConcept, False),
            ("destination", "destination", fhirreference.FHIRReference, False),
            ("dietPreference", "dietPreference", codeableconcept.CodeableConcept, False),
            ("dischargeDiagnosis", "dischargeDiagnosis", fhirreference.FHIRReference, False),
            ("dischargeDisposition", "dischargeDisposition", codeableconcept.CodeableConcept, False),
            ("origin", "origin", fhirreference.FHIRReference, False),
            ("preAdmissionIdentifier", "preAdmissionIdentifier", identifier.Identifier, False),
            ("reAdmission", "reAdmission", bool, False),
            ("specialArrangement", "specialArrangement", codeableconcept.CodeableConcept, True),
            ("specialCourtesy", "specialCourtesy", codeableconcept.CodeableConcept, True),
        ])
        return js


class EncounterLocation(fhirelement.FHIRElement):
    """ List of locations the patient has been at.
    
    List of locations at which the patient has been.
    """
    
    resource_name = "EncounterLocation"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.location = None
        """ Location the encounter takes place.
        Type `FHIRReference` referencing `Location` (represented as `dict` in JSON). """
        
        self.period = None
        """ Time period during which the patient was present at the location.
        Type `Period` (represented as `dict` in JSON). """
        
        self.status = None
        """ planned | present | reserved.
        Type `str`. """
        
        super(EncounterLocation, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(EncounterLocation, self).elementProperties()
        js.extend([
            ("location", "location", fhirreference.FHIRReference, False),
            ("period", "period", period.Period, False),
            ("status", "status", str, False),
        ])
        return js


class EncounterParticipant(fhirelement.FHIRElement):
    """ List of participants involved in the encounter.
    
    The main practitioner responsible for providing the service.
    """
    
    resource_name = "EncounterParticipant"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.individual = None
        """ Persons involved in the encounter other than the patient.
        Type `FHIRReference` referencing `Practitioner, RelatedPerson` (represented as `dict` in JSON). """
        
        self.period = None
        """ Period of time during the encounter participant was present.
        Type `Period` (represented as `dict` in JSON). """
        
        self.type = None
        """ Role of participant in encounter.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        super(EncounterParticipant, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(EncounterParticipant, self).elementProperties()
        js.extend([
            ("individual", "individual", fhirreference.FHIRReference, False),
            ("period", "period", period.Period, False),
            ("type", "type", codeableconcept.CodeableConcept, True),
        ])
        return js


class EncounterStatusHistory(fhirelement.FHIRElement):
    """ List of Encounter statuses.
    
    The current status is always found in the current version of the resource.
    This status history permits the encounter resource to contain the status
    history without the needing to read through the historical versions of the
    resource, or even have the server store them.
    """
    
    resource_name = "EncounterStatusHistory"
    
    def __init__(self, jsondict=None):
        """ Initialize all valid properties.
        """
        
        self.period = None
        """ The time that the episode was in the specified status.
        Type `Period` (represented as `dict` in JSON). """
        
        self.status = None
        """ planned | arrived | in-progress | onleave | finished | cancelled.
        Type `str`. """
        
        super(EncounterStatusHistory, self).__init__(jsondict)
    
    def elementProperties(self):
        js = super(EncounterStatusHistory, self).elementProperties()
        js.extend([
            ("period", "period", period.Period, False),
            ("status", "status", str, False),
        ])
        return js




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 0.5.0.5149 on 2015-07-06.
#  2015, SMART Health IT.


import os
import io
import unittest
import json
from . import searchparameter
from .fhirdate import FHIRDate


class SearchParameterTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("SearchParameter", js["resourceType"])
        return searchparameter.SearchParameter(js)
    
    def testSearchParameter1(self):
        inst = self.instantiate_from("searchparameter-example-extension.json")
        self.assertIsNotNone(inst, "Must have instantiated a SearchParameter instance")
        self.implSearchParameter1(inst)
        
        js = inst.as_json()
        self.assertEqual("SearchParameter", js["resourceType"])
        inst2 = searchparameter.SearchParameter(js)
        self.implSearchParameter1(inst2)
    
    def implSearchParameter1(self, inst):
        self.assertEqual(inst.base, "Patient")
        self.assertEqual(inst.contact[0].telecom[0].system, "url")
        self.assertEqual(inst.contact[0].telecom[0].value, "http://hl7.org/fhir")
        self.assertEqual(inst.description, "Search by url for a participation agreement")
        self.assertEqual(inst.id, "example-extension")
        self.assertEqual(inst.name, "Example Search Parameter on an extension")
        self.assertEqual(inst.publisher, "HL7 FHIR Project")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.type, "token")
        self.assertEqual(inst.url, "http://hl7.org/fhir/SearchParameter/example-extension")
        self.assertEqual(inst.xpath, "http://example.org/fhir/StructureDefinition/participation-agreement")
    
    def testSearchParameter2(self):
        inst = self.instantiate_from("searchparameter-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a SearchParameter instance")
        self.implSearchParameter2(inst)
        
        js = inst.as_json()
        self.assertEqual("SearchParameter", js["resourceType"])
        inst2 = searchparameter.SearchParameter(js)
        self.implSearchParameter2(inst2)
    
    def implSearchParameter2(self, inst):
        self.assertEqual(inst.base, "Resource")
        self.assertEqual(inst.contact[0].telecom[0].system, "url")
        self.assertEqual(inst.contact[0].telecom[0].value, "http://hl7.org/fhir")
        self.assertEqual(inst.description, "Search by resource identifier - e.g. same as the read interaction, but can return included resources")
        self.assertEqual(inst.id, "example")
        self.assertEqual(inst.name, "Example Search Parameter")
        self.assertEqual(inst.publisher, "HL7 FHIR Project")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.type, "token")
        self.assertEqual(inst.url, "http://hl7.org/fhir/SearchParameter/example")
        self.assertEqual(inst.xpath, "f:id")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 0.5.0.5149 on 2015-07-06.
#  2015, SMART Health IT.


import os
import io
import unittest
import json
from . import diagnosticorder
from .fhirdate import FHIRDate


class DiagnosticOrderTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("DiagnosticOrder", js["resourceType"])
        return diagnosticorder.DiagnosticOrder(js)
    
    def testDiagnosticOrder1(self):
        inst = self.instantiate_from("diagnosticorder-example-di.json")
        self.assertIsNotNone(inst, "Must have instantiated a DiagnosticOrder instance")
        self.implDiagnosticOrder1(inst)
        
        js = inst.as_json()
        self.assertEqual("DiagnosticOrder", js["resourceType"])
        inst2 = diagnosticorder.DiagnosticOrder(js)
        self.implDiagnosticOrder1(inst2)
    
    def implDiagnosticOrder1(self, inst):
        self.assertEqual(inst.event[0].dateTime.date, FHIRDate("2013-05-08T09:33:27+07:00").date)
        self.assertEqual(inst.event[0].dateTime.as_json(), "2013-05-08T09:33:27+07:00")
        self.assertEqual(inst.event[0].status, "requested")
        self.assertEqual(inst.id, "di")
        self.assertEqual(inst.item[0].code.coding[0].code, "24627-2")
        self.assertEqual(inst.item[0].code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.item[0].code.text, "Chest CT")
        self.assertEqual(inst.status, "requested")
        self.assertEqual(inst.text.status, "generated")
    
    def testDiagnosticOrder2(self):
        inst = self.instantiate_from("diagnosticorder-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a DiagnosticOrder instance")
        self.implDiagnosticOrder2(inst)
        
        js = inst.as_json()
        self.assertEqual("DiagnosticOrder", js["resourceType"])
        inst2 = diagnosticorder.DiagnosticOrder(js)
        self.implDiagnosticOrder2(inst2)
    
    def implDiagnosticOrder2(self, inst):
        self.assertEqual(inst.event[0].dateTime.date, FHIRDate("2013-05-02T16:16:00-07:00").date)
        self.assertEqual(inst.event[0].dateTime.as_json(), "2013-05-02T16:16:00-07:00")
        self.assertEqual(inst.event[0].status, "requested")
        self.assertEqual(inst.id, "example")
        self.assertEqual(inst.identifier[0].system, "urn:oid:1.3.4.5.6.7")
        self.assertEqual(inst.identifier[0].type.text, "Placer")
        self.assertEqual(inst.identifier[0].value, "2345234234234")
        self.assertEqual(inst.item[0].code.coding[0].code, "LIPID")
        self.assertEqual(inst.item[0].code.coding[0].system, "http://acme.org/tests")
        self.assertEqual(inst.item[0].code.text, "Lipid Panel")
        self.assertEqual(inst.status, "received")
        self.assertEqual(inst.text.status, "generated")
    
    def testDiagnosticOrder3(self):
        inst = self.instantiate_from("diagnosticorder-qicore-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a DiagnosticOrder instance")
        self.implDiagnosticOrder3(inst)
        
        js = inst.as_json()
        self.assertEqual("DiagnosticOrder", js["resourceType"])
        inst2 = diagnosticorder.DiagnosticOrder(js)
        self.implDiagnosticOrder3(inst2)
    
    def implDiagnosticOrder3(self, inst):
        self.assertEqual(inst.event[0].dateTime.date, FHIRDate("2013-05-02T16:16:00-07:00").date)
        self.assertEqual(inst.event[0].dateTime.as_json(), "2013-05-02T16:16:00-07:00")
        self.assertEqual(inst.event[0].status, "requested")
        self.assertEqual(inst.event[1].dateTime.date, FHIRDate("2013-05-06T11:20:00-07:00").date)
        self.assertEqual(inst.event[1].dateTime.as_json(), "2013-05-06T11:20:00-07:00")
        self.assertEqual(inst.event[1].status, "rejected")
        self.assertEqual(inst.extension[0].url, "http://hl7.org/fhir/StructureDefinition/diagnosticorder-reason")
        self.assertEqual(inst.extension[0].valueCodeableConcept.coding[0].code, "PHY")
        self.assertEqual(inst.extension[0].valueCodeableConcept.coding[0].display, "Physician request")
        self.assertEqual(inst.extension[0].valueCodeableConcept.coding[0].system, "http://hl7.org/fhir/v3/ActReason")
        self.assertEqual(inst.extension[1].url, "http://hl7.org/fhir/StructureDefinition/diagnosticorder-reasonRejected")
        self.assertEqual(inst.extension[1].valueCodeableConcept.coding[0].code, "NON-AVAIL")
        self.assertEqual(inst.extension[1].valueCodeableConcept.coding[0].display, "patient not-available")
        self.assertEqual(inst.extension[1].valueCodeableConcept.coding[0].system, "http://hl7.org/fhir/v3/ActReason")
        self.assertEqual(inst.id, "qicore")
        self.assertEqual(inst.identifier[0].system, "urn:oid:1.3.4.5.6.7")
        self.assertEqual(inst.identifier[0].type.text, "Placer")
        self.assertEqual(inst.identifier[0].value, "2345234234234")
        self.assertEqual(inst.item[0].code.coding[0].code, "57698-3")
        self.assertEqual(inst.item[0].code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.item[0].code.text, "Lipid panel with direct LDL - Serum or Plasma")
        self.assertEqual(inst.status, "rejected")
        self.assertEqual(inst.text.status, "generated")
    
    def testDiagnosticOrder4(self):
        inst = self.instantiate_from("do-uslab-example1.json")
        self.assertIsNotNone(inst, "Must have instantiated a DiagnosticOrder instance")
        self.implDiagnosticOrder4(inst)
        
        js = inst.as_json()
        self.assertEqual("DiagnosticOrder", js["resourceType"])
        inst2 = diagnosticorder.DiagnosticOrder(js)
        self.implDiagnosticOrder4(inst2)
    
    def implDiagnosticOrder4(self, inst):
        self.assertEqual(inst.clinicalNotes, "Screening for blood lead")
        self.assertEqual(inst.event[0].dateTime.date, FHIRDate("2014-12-04T15:42:15-08:00").date)
        self.assertEqual(inst.event[0].dateTime.as_json(), "2014-12-04T15:42:15-08:00")
        self.assertEqual(inst.event[0].description.coding[0].code, "new-request")
        self.assertEqual(inst.event[0].status, "requested")
        self.assertEqual(inst.id, "uslab-example1")
        self.assertEqual(inst.identifier[0].system, "urn:oid:2.16.840.1.113883.3.72.5.24")
        self.assertEqual(inst.identifier[0].use, "official")
        self.assertEqual(inst.identifier[0].value, "ORD000123A")
        self.assertEqual(inst.item[0].code.coding[0].code, "5671-3")
        self.assertEqual(inst.item[0].code.coding[0].display, "Lead [Mass/volume] in Blood")
        self.assertEqual(inst.item[0].code.coding[0].system, "http://loinc.org")
        self.assertEqual(inst.item[0].code.coding[1].code, "BLDLD")
        self.assertEqual(inst.item[0].code.coding[1].display, "Blood Lead")
        self.assertEqual(inst.item[0].code.coding[1].system, "urn:oid:2.16.840.1.113883.3.72.5.24")
        self.assertEqual(inst.item[0].code.text, "Blood Lead")
        self.assertEqual(inst.item[0].status, "requested")
        self.assertEqual(inst.priority, "routine")
        self.assertEqual(inst.status, "requested")
        self.assertEqual(inst.text.status, "generated")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 0.5.0.5149 on 2015-07-06.
#  2015, SMART Health IT.


import os
import io
import unittest
import json
from . import communicationrequest
from .fhirdate import FHIRDate


class CommunicationRequestTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("CommunicationRequest", js["resourceType"])
        return communicationrequest.CommunicationRequest(js)
    
    def testCommunicationRequest1(self):
        inst = self.instantiate_from("communicationrequest-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a CommunicationRequest instance")
        self.implCommunicationRequest1(inst)
        
        js = inst.as_json()
        self.assertEqual("CommunicationRequest", js["resourceType"])
        inst2 = communicationrequest.CommunicationRequest(js)
        self.implCommunicationRequest1(inst2)
    
    def implCommunicationRequest1(self, inst):
        self.assertEqual(inst.id, "example")
        self.assertEqual(inst.text.div, "<div>To be filled out at a later time</div>")
        self.assertEqual(inst.text.status, "generated")
    
    def testCommunicationRequest2(self):
        inst = self.instantiate_from("communicationrequest-qicore-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a CommunicationRequest instance")
        self.implCommunicationRequest2(inst)
        
        js = inst.as_json()
        self.assertEqual("CommunicationRequest", js["resourceType"])
        inst2 = communicationrequest.CommunicationRequest(js)
        self.implCommunicationRequest2(inst2)
    
    def implCommunicationRequest2(self, inst):
        self.assertEqual(inst.extension[0].url, "http://hl7.org/fhir/StructureDefinition/communicationrequest-orderedBy")
        self.assertEqual(inst.extension[1].url, "http://hl7.org/fhir/StructureDefinition/communicationrequest-reasonRejected")
        self.assertEqual(inst.extension[1].valueCodeableConcept.coding[0].code, "NON-AVAIL")
        self.assertEqual(inst.extension[1].valueCodeableConcept.coding[0].display, "patient not-available")
        self.assertEqual(inst.extension[1].valueCodeableConcept.coding[0].system, "http://hl7.org/fhir/v3/ActReason")
        self.assertEqual(inst.id, "qicore")
        self.assertEqual(inst.status, "rejected")
        self.assertEqual(inst.text.div, "<div>To be filled out at a later time</div>")
        self.assertEqual(inst.text.status, "generated")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 0.5.0.5149 on 2015-07-06.
#  2015, SMART Health IT.


import os
import io
import unittest
import json
from . import dataelement
from .fhirdate import FHIRDate


class DataElementTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("DataElement", js["resourceType"])
        return dataelement.DataElement(js)
    
    def testDataElement1(self):
        inst = self.instantiate_from("dataelement-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a DataElement instance")
        self.implDataElement1(inst)
        
        js = inst.as_json()
        self.assertEqual("DataElement", js["resourceType"])
        inst2 = dataelement.DataElement(js)
        self.implDataElement1(inst2)
    
    def implDataElement1(self, inst):
        self.assertEqual(inst.element[0].binding.name, "Gender")
        self.assertEqual(inst.element[0].binding.strength, "required")
        self.assertEqual(inst.element[0].binding.valueSetReference.extension[0].url, "http://hl7.org/fhir/StructureDefinition/11179-permitted-value-valueset")
        self.assertEqual(inst.element[0].binding.valueSetReference.extension[1].url, "http://hl7.org/fhir/StructureDefinition/11179-permitted-value-conceptmap")
        self.assertEqual(inst.element[0].definition, "The code representing the gender of a person.")
        self.assertEqual(inst.element[0].extension[0].url, "http://hl7.org/fhir/StructureDefinition/minLength")
        self.assertEqual(inst.element[0].extension[0].valueInteger, 1)
        self.assertEqual(inst.element[0].extension[1].url, "http://hl7.org/fhir/StructureDefinition/elementdefinition-question")
        self.assertEqual(inst.element[0].extension[1].valueString, "Gender")
        self.assertEqual(inst.element[0].maxLength, 13)
        self.assertEqual(inst.element[0].path, "Gender")
        self.assertEqual(inst.element[0].type[0].code, "CodeableConcept")
        self.assertEqual(inst.id, "gender")
        self.assertEqual(inst.identifier.value, "2179650")
        self.assertEqual(inst.name, "Gender Code")
        self.assertEqual(inst.publisher, "DCP")
        self.assertEqual(inst.status, "active")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.useContext[0].coding[0].display, "FBPP Pooled Database")
        self.assertEqual(inst.useContext[0].coding[0].system, "http://example.org/FBPP")
        self.assertEqual(inst.useContext[0].coding[1].display, "Demographics")
        self.assertEqual(inst.useContext[0].coding[1].system, "http://example.org/PhenX")
        self.assertEqual(inst.useContext[0].coding[2].display, "Pt. Administrative")
        self.assertEqual(inst.useContext[0].coding[2].system, "http://example.org/EligibilityCriteria")
        self.assertEqual(inst.useContext[0].coding[3].display, "UAMS New CDEs")
        self.assertEqual(inst.useContext[0].coding[3].system, "http://example.org/UAMSClinicalResearch")
        self.assertEqual(inst.useContext[0].coding[4].display, "Substance Abuse and ")
        self.assertEqual(inst.useContext[0].coding[4].system, "http://example.org/PhenX")
        self.assertEqual(inst.useContext[0].coding[5].display, "CSAERS Adverse Event")
        self.assertEqual(inst.useContext[0].coding[5].system, "http://example.org/Category")
        self.assertEqual(inst.useContext[0].coding[6].display, "Core: Tier 1")
        self.assertEqual(inst.useContext[0].coding[6].system, "http://example.org/PhenX")
        self.assertEqual(inst.useContext[0].coding[7].display, "Case Report Forms")
        self.assertEqual(inst.useContext[0].coding[7].system, "http://example.org/Category")
        self.assertEqual(inst.useContext[0].coding[8].display, "CSAERS Review Set")
        self.assertEqual(inst.useContext[0].coding[8].system, "http://example.org/Category")
        self.assertEqual(inst.useContext[0].coding[9].display, "CIAF")
        self.assertEqual(inst.useContext[0].coding[9].system, "http://example.org/Demonstration%20Applications")
        self.assertEqual(inst.version, "1.0")
    
    def testDataElement2(self):
        inst = self.instantiate_from("dataelement-labtestmaster-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a DataElement instance")
        self.implDataElement2(inst)
        
        js = inst.as_json()
        self.assertEqual("DataElement", js["resourceType"])
        inst2 = dataelement.DataElement(js)
        self.implDataElement2(inst2)
    
    def implDataElement2(self, inst):
        self.assertEqual(inst.element[0].alias[0], "Protime, PT")
        self.assertEqual(inst.element[0].comments, "Used to screen the integrity of the extrinsic and common pathways of coagulation and to monitor warfarin anticoagulation. ")
        self.assertEqual(inst.element[0].definition, "The PT test evaluates the extrinsic and common pathways of the coagulation cascade.")
        self.assertEqual(inst.element[0].exampleDecimal, 10.0)
        self.assertEqual(inst.element[0].extension[0].url, "http://hl7.org/fhir/StructureDefinition/elementdefinition-allowedUnits")
        self.assertEqual(inst.element[0].extension[0].valueCodeableConcept.coding[0].code, "s")
        self.assertEqual(inst.element[0].extension[0].valueCodeableConcept.coding[0].display, "second")
        self.assertTrue(inst.element[0].extension[0].valueCodeableConcept.coding[0].primary)
        self.assertEqual(inst.element[0].extension[0].valueCodeableConcept.coding[0].system, "http://unitsofmeasure.org")
        self.assertEqual(inst.element[0].extension[0].valueCodeableConcept.coding[0].version, "1.9")
        self.assertEqual(inst.element[0].extension[0].valueCodeableConcept.text, "second")
        self.assertEqual(inst.element[0].mapping[0].identity, "loinc")
        self.assertEqual(inst.element[0].mapping[0].map, "5964-2")
        self.assertEqual(inst.element[0].path, "prothrombin")
        self.assertEqual(inst.element[0].requirements, "This test is orderable. A plasma specimen in a 3.2% sodium citrate blue top tube is required.")
        self.assertEqual(inst.element[0].type[0].code, "Decimal")
        self.assertEqual(inst.id, "prothrombin")
        self.assertEqual(inst.identifier.period.start.date, FHIRDate("2011-05-19").date)
        self.assertEqual(inst.identifier.period.start.as_json(), "2011-05-19")
        self.assertEqual(inst.identifier.system, "http://www.CenturyHospital/Laboratory/DirectoryofServices")
        self.assertEqual(inst.identifier.type.text, "Prothrombin Time, PT")
        self.assertEqual(inst.identifier.value, "11")
        self.assertEqual(inst.mapping[0].comments, "Version 2.48 or later")
        self.assertEqual(inst.mapping[0].identity, "loinc")
        self.assertEqual(inst.mapping[0].name, "LOINC")
        self.assertEqual(inst.mapping[0].uri, "http://loinc.org/")
        self.assertEqual(inst.name, "Prothrombin Time")
        self.assertEqual(inst.status, "active")
        self.assertEqual(inst.text.status, "generated")
    
    def testDataElement3(self):
        inst = self.instantiate_from("dataelement-sdc-profile-example-de.json")
        self.assertIsNotNone(inst, "Must have instantiated a DataElement instance")
        self.implDataElement3(inst)
        
        js = inst.as_json()
        self.assertEqual("DataElement", js["resourceType"])
        inst2 = dataelement.DataElement(js)
        self.implDataElement3(inst2)
    
    def implDataElement3(self, inst):
        self.assertEqual(inst.contact[0].extension[0].url, "http://hl7.org/fhir/StructureDefinition/11179-de-contact-address")
        self.assertEqual(inst.contact[0].extension[0].valueAddress.text, "540 Gaither Rd Rockville MD 20850")
        self.assertEqual(inst.contact[0].name, "Bill Munier")
        self.assertEqual(inst.contact[0].telecom[0].system, "email")
        self.assertEqual(inst.contact[0].telecom[0].value, "william.munier@ahrq.hhs.gov")
        self.assertEqual(inst.date.date, FHIRDate("2010-03-31").date)
        self.assertEqual(inst.date.as_json(), "2010-03-31")
        self.assertEqual(inst.element[0].binding.name, "Gender")
        self.assertEqual(inst.element[0].binding.strength, "required")
        self.assertEqual(inst.element[0].binding.valueSetReference.extension[0].url, "http://hl7.org/fhir/StructureDefinition/11179-permitted-value-valueset")
        self.assertEqual(inst.element[0].binding.valueSetReference.extension[1].url, "http://hl7.org/fhir/StructureDefinition/11179-permitted-value-valueset")
        self.assertEqual(inst.element[0].code[0].code, "74698-2")
        self.assertEqual(inst.element[0].code[0].extension[0].url, "http://hl7.org/fhir/StructureDefinition/11179-de-is-data-element-concept")
        self.assertTrue(inst.element[0].code[0].extension[0].valueBoolean)
        self.assertEqual(inst.element[0].code[0].system, "http://loinc.org")
        self.assertEqual(inst.element[0].definition, "The patient's documented gender on arrival at the hospital.")
        self.assertEqual(inst.element[0].extension[0].url, "http://hl7.org/fhir/StructureDefinition/elementdefinition-question")
        self.assertEqual(inst.element[0].extension[0].valueString, "Patient's Gender")
        self.assertEqual(inst.element[0].extension[1].url, "http://hl7.org/fhir/StructureDefinition/elementdefinition-question")
        self.assertEqual(inst.element[0].extension[1].valueString, "Patient-Gender,CD")
        self.assertEqual(inst.element[0].mapping[0].extension[0].url, "http://hl7.org/fhir/StructureDefinition/11179-objectClass")
        self.assertEqual(inst.element[0].mapping[0].extension[0].valueCoding.code, "116154003")
        self.assertEqual(inst.element[0].mapping[0].extension[0].valueCoding.display, "Patient")
        self.assertEqual(inst.element[0].mapping[0].extension[0].valueCoding.system, "http://snomed.info/sct")
        self.assertEqual(inst.element[0].mapping[0].extension[1].url, "http://hl7.org/fhir/StructureDefinition/11179-objectClassProperty")
        self.assertEqual(inst.element[0].mapping[0].extension[1].valueCoding.code, "263495000")
        self.assertEqual(inst.element[0].mapping[0].extension[1].valueCoding.display, "Gender")
        self.assertEqual(inst.element[0].mapping[0].extension[1].valueCoding.system, "http://snomed.info/sct")
        self.assertEqual(inst.element[0].mapping[0].identity, "objclass")
        self.assertEqual(inst.element[0].mapping[0].map, "http://snomed.info/sct#116154003 | http://snomed.info/sct | 263495000")
        self.assertEqual(inst.element[0].path, "gender")
        self.assertEqual(inst.element[0].type[0].code, "CodeableConcept")
        self.assertEqual(inst.extension[0].url, "http://hl7.org/fhir/StructureDefinition/11179-de-document-reference")
        self.assertEqual(inst.extension[1].url, "http://hl7.org/fhir/StructureDefinition/11179-de-effective-period")
        self.assertEqual(inst.extension[1].valuePeriod.start.date, FHIRDate("2010-03-31").date)
        self.assertEqual(inst.extension[1].valuePeriod.start.as_json(), "2010-03-31")
        self.assertEqual(inst.extension[2].url, "http://hl7.org/fhir/StructureDefinition/11179-de-submitter-org")
        self.assertEqual(inst.id, "sdc-gender-de")
        self.assertEqual(inst.identifier.system, "http://nlm.nih.gov/some_other_text/data_element_identifier")
        self.assertEqual(inst.identifier.value, "DE42AHRQ")
        self.assertEqual(inst.mapping[0].identity, "objclass")
        self.assertEqual(inst.mapping[0].name, "Structured Data Capture ObjectClass/Property")
        self.assertEqual(inst.mapping[0].uri, "http://healthit.gov/sdc/objectclassproperty")
        self.assertEqual(inst.name, "Patient Gender")
        self.assertEqual(inst.publisher, "Agency for Healthcare Research and Quality (AHRQ) Center for Patient Safety and Quality")
        self.assertEqual(inst.status, "draft")
        self.assertEqual(inst.text.div, "<div>TODO</div>")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.useContext[0].coding[0].code, "Healthcare Event Reporting Form")
        self.assertEqual(inst.useContext[0].coding[0].display, "Patient safety event report - hospital: healthcare event reporting form")
        self.assertEqual(inst.version, "1.2")
    
    def testDataElement4(self):
        inst = self.instantiate_from("dataelement-sdc-profile-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a DataElement instance")
        self.implDataElement4(inst)
        
        js = inst.as_json()
        self.assertEqual("DataElement", js["resourceType"])
        inst2 = dataelement.DataElement(js)
        self.implDataElement4(inst2)
    
    def implDataElement4(self, inst):
        self.assertEqual(inst.contact[0].telecom[0].system, "url")
        self.assertEqual(inst.contact[0].telecom[0].value, "http://hl7.org")
        self.assertEqual(inst.date.date, FHIRDate("2015-02-04").date)
        self.assertEqual(inst.date.as_json(), "2015-02-04")
        self.assertEqual(inst.element[0].binding.name, "AdministrativeGender")
        self.assertEqual(inst.element[0].binding.strength, "required")
        self.assertEqual(inst.element[0].definition, "The gender (i.e., the behavioral, cultural, or psychological traits typically associated with one sex) of a living subject as defined for administrative purposes.")
        self.assertEqual(inst.element[0].mapping[0].identity, "ccda")
        self.assertEqual(inst.element[0].mapping[0].language, "xpath")
        self.assertEqual(inst.element[0].mapping[0].map, "/ClinicalDocument/recordTarget/patientRole/patient/administrativeGender")
        self.assertEqual(inst.element[0].path, "administrativeGender")
        self.assertEqual(inst.element[0].type[0].code, "CodeableConcept")
        self.assertEqual(inst.id, "sdc-gender")
        self.assertEqual(inst.identifier.system, "http://nlm.nih.gov/some_other_text/data_element_identifier")
        self.assertEqual(inst.identifier.value, "DE42AHRQ")
        self.assertEqual(inst.mapping[0].identity, "ccda")
        self.assertEqual(inst.mapping[0].name, "Consolidated CDA R2")
        self.assertEqual(inst.mapping[0].uri, "http://hl7.org/CCDA-not-a-real-url")
        self.assertEqual(inst.name, "Patient Gender")
        self.assertEqual(inst.publisher, "Health Level Seven, International")
        self.assertEqual(inst.status, "draft")
        self.assertEqual(inst.text.div, "<div>TODO</div>")
        self.assertEqual(inst.text.status, "generated")

